# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Dragonfly(KaitaiStruct):
    """:field cw_name_callsign: id1.id2.cw_name_callsign
    :field bat_v: id1.id2.bat_v
    :field bat_i: id1.id2.bat_i
    :field bat_t: id1.id2.bat_t
    :field kill_main: id1.id2.kill_main
    :field kill_fab: id1.id2.kill_fab
    :field solar_cell_plus_x: id1.id2.solar_cell_plus_x
    :field solar_cell_plus_y: id1.id2.solar_cell_plus_y
    :field solar_cell_plus_z: id1.id2.solar_cell_plus_z
    :field solar_cell_minus_x: id1.id2.solar_cell_minus_x
    :field solar_cell_minus_y: id1.id2.solar_cell_minus_y
    :field solar_cell_minus_z: id1.id2.solar_cell_minus_z
    :field ant_1_deploy: id1.id2.ant_1_deploy
    :field ant_1_set_count: id1.id2.ant_1_set_count
    :field ant_2_deploy: id1.id2.ant_2_deploy
    :field ant_2_set_count: id1.id2.ant_2_set_count
    :field aprs_reference_1: id1.id2.aprs_reference_1
    :field aprs_reference_2: id1.id2.aprs_reference_2
    :field aprs_payload_1: id1.id2.aprs_payload_1
    :field aprs_payload_2: id1.id2.aprs_payload_2
    :field aprs_payload_3: id1.id2.aprs_payload_3
    :field aprs_payload_4: id1.id2.aprs_payload_4
    :field aprs_payload_5: id1.id2.aprs_payload_5
    :field main_pic_power_line_status: id1.id2.main_pic_power_line_status
    :field com_pic_power_line_status: id1.id2.com_pic_power_line_status
    :field v3_3_1_status: id1.id2.v3_3_1_status
    :field v3_3_2_status: id1.id2.v3_3_2_status
    :field v5_status: id1.id2.v5_status
    :field unreg1_status: id1.id2.unreg1_status
    :field unreg2_status: id1.id2.unreg2_status
    :field time_after_last_reset: id1.id2.time_after_last_reset
    :field necessary_for_lengthcheck: id1.id2.necessary_for_lengthcheck
    :field beacon_type: id1.id2.beacon_type
    :field cw_beacon: id1.id2.cw_beacon
    
    :field packet_number: id1.id2.id3.packet_number
    :field bat_v: id1.id2.id3.bat_v
    :field bat_i: id1.id2.id3.bat_i
    :field bat_t: id1.id2.id3.bat_t
    :field kill_main: id1.id2.id3.kill_main
    :field kill_fab: id1.id2.id3.kill_fab
    :field solar_cell_plus_x: id1.id2.id3.solar_cell_plus_x
    :field solar_cell_plus_y: id1.id2.id3.solar_cell_plus_y
    :field solar_cell_plus_z: id1.id2.id3.solar_cell_plus_z
    :field solar_cell_minus_x: id1.id2.id3.solar_cell_minus_x
    :field solar_cell_minus_y: id1.id2.id3.solar_cell_minus_y
    :field solar_cell_minus_z: id1.id2.id3.solar_cell_minus_z
    :field ant_1_deploy: id1.id2.id3.ant_1_deploy
    :field ant_1_set_count: id1.id2.id3.ant_1_set_count
    :field ant_2_deploy: id1.id2.id3.ant_2_deploy
    :field ant_2_set_count: id1.id2.id3.ant_2_set_count
    :field aprs_reference_1: id1.id2.id3.aprs_reference_1
    :field aprs_reference_2: id1.id2.id3.aprs_reference_2
    :field aprs_payload_1: id1.id2.id3.aprs_payload_1
    :field aprs_payload_2: id1.id2.id3.aprs_payload_2
    :field aprs_payload_3: id1.id2.id3.aprs_payload_3
    :field aprs_payload_4: id1.id2.id3.aprs_payload_4
    :field aprs_payload_5: id1.id2.id3.aprs_payload_5
    :field main_pic_power_line_status: id1.id2.id3.main_pic_power_line_status
    :field com_pic_power_line_status: id1.id2.id3.com_pic_power_line_status
    :field v3_3_1_status: id1.id2.id3.v3_3_1_status
    :field v3_3_2_status: id1.id2.id3.v3_3_2_status
    :field v5_status: id1.id2.id3.v5_status
    :field unreg1_status: id1.id2.id3.unreg1_status
    :field unreg2_status: id1.id2.id3.unreg2_status
    :field time_after_last_reset: id1.id2.id3.time_after_last_reset
    :field necessary_for_lengthcheck: id1.id2.id3.necessary_for_lengthcheck
    :field beacon_type: id1.id2.id3.beacon_type
    
    :field digi_dest_callsign: id1.id2.id3.ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field digi_src_callsign: id1.id2.id3.ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field digi_src_ssid: id1.id2.id3.ax25_frame.ax25_header.src_ssid_raw.ssid
    :field digi_dest_ssid: id1.id2.id3.ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field rpt_instance___callsign: id1.id2.id3.ax25_frame.ax25_header.repeater.rpt_instance.___.rpt_callsign_raw.callsign_ror.callsign
    :field rpt_instance___ssid: id1.id2.id3.ax25_frame.ax25_header.repeater.rpt_instance.___.rpt_ssid_raw.ssid
    :field rpt_instance___hbit: id1.id2.id3.ax25_frame.ax25_header.repeater.rpt_instance.___.rpt_ssid_raw.hbit
    :field digi_ctl: id1.id2.id3.ax25_frame.ax25_header.ctl
    :field digi_pid: id1.id2.id3.ax25_frame.ax25_header.pid
    :field digi_message: id1.id2.id3.ax25_frame.ax25_info.digi_message
    
    .. seealso::
       Source - https://birds-x.birds-project.com/satellite-information/
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.id1 = Dragonfly.Type1(self._io, self, self._root)

    class Type1(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            _on = self.message_type1
            if _on == 7237954648016840300:
                self.id2 = Dragonfly.Cw(self._io, self, self._root)
            else:
                self.id2 = Dragonfly.GmskOrDigi(self._io, self, self._root)

        @property
        def message_type1(self):
            if hasattr(self, '_m_message_type1'):
                return self._m_message_type1

            _pos = self._io.pos()
            self._io.seek(0)
            self._m_message_type1 = self._io.read_u8be()
            self._io.seek(_pos)
            return getattr(self, '_m_message_type1', None)


    class Gmsk(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.header_and_reserved_1 = self._io.read_u8be()
            self.header_and_reserved_2 = self._io.read_u8be()
            self.header_and_reserved_3 = self._io.read_bits_int_be(24)
            self.packet_number = self._io.read_bits_int_be(24)
            self._io.align_to_byte()
            self.bat_v_raw_1 = self._io.read_u1()
            self.bat_v_raw_2 = self._io.read_u1()
            self.bat_i_raw_1 = self._io.read_u1()
            self.bat_i_raw_2 = self._io.read_u1()
            self.bat_t_raw_1 = self._io.read_u1()
            self.bat_t_raw_2 = self._io.read_u1()
            self.free_1 = self._io.read_bits_int_be(4)
            self.kill_main_boolean = self._io.read_bits_int_be(1) != 0
            self.kill_fab_boolean = self._io.read_bits_int_be(1) != 0
            self.solar_cell_plus_x_boolean = self._io.read_bits_int_be(1) != 0
            self.solar_cell_plus_y_boolean = self._io.read_bits_int_be(1) != 0
            self.free_2 = self._io.read_bits_int_be(4)
            self.solar_cell_plus_z_boolean = self._io.read_bits_int_be(1) != 0
            self.solar_cell_minus_x_boolean = self._io.read_bits_int_be(1) != 0
            self.solar_cell_minus_y_boolean = self._io.read_bits_int_be(1) != 0
            self.solar_cell_minus_z_boolean = self._io.read_bits_int_be(1) != 0
            self.free_3 = self._io.read_bits_int_be(4)
            self.ant_1_deploy_boolean = self._io.read_bits_int_be(1) != 0
            self.ant_1_set_count = self._io.read_bits_int_be(3)
            self.free_4 = self._io.read_bits_int_be(4)
            self.ant_2_deploy_boolean = self._io.read_bits_int_be(1) != 0
            self.ant_2_set_count = self._io.read_bits_int_be(3)
            self.free_5 = self._io.read_bits_int_be(4)
            self.aprs_reference_1_boolean = self._io.read_bits_int_be(1) != 0
            self.aprs_reference_2_boolean = self._io.read_bits_int_be(1) != 0
            self.aprs_payload_1_boolean = self._io.read_bits_int_be(1) != 0
            self.aprs_payload_2_boolean = self._io.read_bits_int_be(1) != 0
            self.free_6 = self._io.read_bits_int_be(4)
            self.aprs_payload_3_boolean = self._io.read_bits_int_be(1) != 0
            self.aprs_payload_4_boolean = self._io.read_bits_int_be(1) != 0
            self.aprs_payload_5_boolean = self._io.read_bits_int_be(1) != 0
            self.free_7 = self._io.read_bits_int_be(5)
            self.main_pic_power_line_status_boolean = self._io.read_bits_int_be(1) != 0
            self.com_pic_power_line_status_boolean = self._io.read_bits_int_be(1) != 0
            self.v3_3_1_status_boolean = self._io.read_bits_int_be(1) != 0
            self.v3_3_2_status_boolean = self._io.read_bits_int_be(1) != 0
            self.free_8 = self._io.read_bits_int_be(4)
            self.v5_status_boolean = self._io.read_bits_int_be(1) != 0
            self.unreg1_status_boolean = self._io.read_bits_int_be(1) != 0
            self.unreg2_status_boolean = self._io.read_bits_int_be(1) != 0
            self._io.align_to_byte()
            self.time_after_last_reset_1 = self._io.read_u1()
            self.time_after_last_reset_2 = self._io.read_u1()
            self.ff = []
            for i in range(65):
                self.ff.append(self._io.read_u1())

            self.lengthcheck = (self._io.read_bytes_full()).decode(u"utf-8")

        @property
        def bat_i_raw(self):
            if hasattr(self, '_m_bat_i_raw'):
                return self._m_bat_i_raw

            self._m_bat_i_raw = ((self.bat_i_raw_1 << 4) + self.bat_i_raw_2)
            return getattr(self, '_m_bat_i_raw', None)

        @property
        def bat_v(self):
            if hasattr(self, '_m_bat_v'):
                return self._m_bat_v

            self._m_bat_v = ((6.512 * self.bat_v_raw) / 256)
            return getattr(self, '_m_bat_v', None)

        @property
        def main_pic_power_line_status(self):
            if hasattr(self, '_m_main_pic_power_line_status'):
                return self._m_main_pic_power_line_status

            self._m_main_pic_power_line_status = int(self.main_pic_power_line_status_boolean)
            return getattr(self, '_m_main_pic_power_line_status', None)

        @property
        def necessary_for_lengthcheck(self):
            if hasattr(self, '_m_necessary_for_lengthcheck'):
                return self._m_necessary_for_lengthcheck

            if len(self.lengthcheck) != 0:
                self._m_necessary_for_lengthcheck = int(self.lengthcheck) // 0

            return getattr(self, '_m_necessary_for_lengthcheck', None)

        @property
        def aprs_payload_3(self):
            if hasattr(self, '_m_aprs_payload_3'):
                return self._m_aprs_payload_3

            self._m_aprs_payload_3 = int(self.aprs_payload_3_boolean)
            return getattr(self, '_m_aprs_payload_3', None)

        @property
        def unreg1_status(self):
            if hasattr(self, '_m_unreg1_status'):
                return self._m_unreg1_status

            self._m_unreg1_status = int(self.unreg1_status_boolean)
            return getattr(self, '_m_unreg1_status', None)

        @property
        def kill_main(self):
            if hasattr(self, '_m_kill_main'):
                return self._m_kill_main

            self._m_kill_main = int(self.kill_main_boolean)
            return getattr(self, '_m_kill_main', None)

        @property
        def unreg2_status(self):
            if hasattr(self, '_m_unreg2_status'):
                return self._m_unreg2_status

            self._m_unreg2_status = int(self.unreg2_status_boolean)
            return getattr(self, '_m_unreg2_status', None)

        @property
        def bat_v_raw(self):
            if hasattr(self, '_m_bat_v_raw'):
                return self._m_bat_v_raw

            self._m_bat_v_raw = ((self.bat_v_raw_1 << 4) + self.bat_v_raw_2)
            return getattr(self, '_m_bat_v_raw', None)

        @property
        def solar_cell_plus_y(self):
            if hasattr(self, '_m_solar_cell_plus_y'):
                return self._m_solar_cell_plus_y

            self._m_solar_cell_plus_y = int(self.solar_cell_plus_y_boolean)
            return getattr(self, '_m_solar_cell_plus_y', None)

        @property
        def aprs_reference_1(self):
            if hasattr(self, '_m_aprs_reference_1'):
                return self._m_aprs_reference_1

            self._m_aprs_reference_1 = int(self.aprs_reference_1_boolean)
            return getattr(self, '_m_aprs_reference_1', None)

        @property
        def aprs_payload_5(self):
            if hasattr(self, '_m_aprs_payload_5'):
                return self._m_aprs_payload_5

            self._m_aprs_payload_5 = int(self.aprs_payload_5_boolean)
            return getattr(self, '_m_aprs_payload_5', None)

        @property
        def ant_2_deploy(self):
            if hasattr(self, '_m_ant_2_deploy'):
                return self._m_ant_2_deploy

            self._m_ant_2_deploy = int(self.ant_2_deploy_boolean)
            return getattr(self, '_m_ant_2_deploy', None)

        @property
        def aprs_payload_2(self):
            if hasattr(self, '_m_aprs_payload_2'):
                return self._m_aprs_payload_2

            self._m_aprs_payload_2 = int(self.aprs_payload_2_boolean)
            return getattr(self, '_m_aprs_payload_2', None)

        @property
        def v5_status(self):
            if hasattr(self, '_m_v5_status'):
                return self._m_v5_status

            self._m_v5_status = int(self.v5_status_boolean)
            return getattr(self, '_m_v5_status', None)

        @property
        def com_pic_power_line_status(self):
            if hasattr(self, '_m_com_pic_power_line_status'):
                return self._m_com_pic_power_line_status

            self._m_com_pic_power_line_status = int(self.com_pic_power_line_status_boolean)
            return getattr(self, '_m_com_pic_power_line_status', None)

        @property
        def beacon_type(self):
            if hasattr(self, '_m_beacon_type'):
                return self._m_beacon_type

            self._m_beacon_type = u"GMSK"
            return getattr(self, '_m_beacon_type', None)

        @property
        def solar_cell_plus_z(self):
            if hasattr(self, '_m_solar_cell_plus_z'):
                return self._m_solar_cell_plus_z

            self._m_solar_cell_plus_z = int(self.solar_cell_plus_z_boolean)
            return getattr(self, '_m_solar_cell_plus_z', None)

        @property
        def solar_cell_minus_x(self):
            if hasattr(self, '_m_solar_cell_minus_x'):
                return self._m_solar_cell_minus_x

            self._m_solar_cell_minus_x = int(self.solar_cell_minus_x_boolean)
            return getattr(self, '_m_solar_cell_minus_x', None)

        @property
        def bat_i(self):
            if hasattr(self, '_m_bat_i'):
                return self._m_bat_i

            self._m_bat_i = ((-2.99589 * self.bat_i_raw) + 6129.78533)
            return getattr(self, '_m_bat_i', None)

        @property
        def aprs_reference_2(self):
            if hasattr(self, '_m_aprs_reference_2'):
                return self._m_aprs_reference_2

            self._m_aprs_reference_2 = int(self.aprs_reference_2_boolean)
            return getattr(self, '_m_aprs_reference_2', None)

        @property
        def ant_1_deploy(self):
            if hasattr(self, '_m_ant_1_deploy'):
                return self._m_ant_1_deploy

            self._m_ant_1_deploy = int(self.ant_1_deploy_boolean)
            return getattr(self, '_m_ant_1_deploy', None)

        @property
        def v3_3_1_status(self):
            if hasattr(self, '_m_v3_3_1_status'):
                return self._m_v3_3_1_status

            self._m_v3_3_1_status = int(self.v3_3_1_status_boolean)
            return getattr(self, '_m_v3_3_1_status', None)

        @property
        def bat_t_raw(self):
            if hasattr(self, '_m_bat_t_raw'):
                return self._m_bat_t_raw

            self._m_bat_t_raw = ((self.bat_t_raw_1 << 4) + self.bat_t_raw_2)
            return getattr(self, '_m_bat_t_raw', None)

        @property
        def time_after_last_reset(self):
            if hasattr(self, '_m_time_after_last_reset'):
                return self._m_time_after_last_reset

            self._m_time_after_last_reset = ((self.time_after_last_reset_1 << 4) + self.time_after_last_reset_2)
            return getattr(self, '_m_time_after_last_reset', None)

        @property
        def solar_cell_plus_x(self):
            if hasattr(self, '_m_solar_cell_plus_x'):
                return self._m_solar_cell_plus_x

            self._m_solar_cell_plus_x = int(self.solar_cell_plus_x_boolean)
            return getattr(self, '_m_solar_cell_plus_x', None)

        @property
        def kill_fab(self):
            if hasattr(self, '_m_kill_fab'):
                return self._m_kill_fab

            self._m_kill_fab = int(self.kill_fab_boolean)
            return getattr(self, '_m_kill_fab', None)

        @property
        def bat_t(self):
            if hasattr(self, '_m_bat_t'):
                return self._m_bat_t

            self._m_bat_t = (75 - ((self.bat_t_raw * 97.68) / 256))
            return getattr(self, '_m_bat_t', None)

        @property
        def solar_cell_minus_y(self):
            if hasattr(self, '_m_solar_cell_minus_y'):
                return self._m_solar_cell_minus_y

            self._m_solar_cell_minus_y = int(self.solar_cell_minus_y_boolean)
            return getattr(self, '_m_solar_cell_minus_y', None)

        @property
        def aprs_payload_4(self):
            if hasattr(self, '_m_aprs_payload_4'):
                return self._m_aprs_payload_4

            self._m_aprs_payload_4 = int(self.aprs_payload_4_boolean)
            return getattr(self, '_m_aprs_payload_4', None)

        @property
        def solar_cell_minus_z(self):
            if hasattr(self, '_m_solar_cell_minus_z'):
                return self._m_solar_cell_minus_z

            self._m_solar_cell_minus_z = int(self.solar_cell_minus_z_boolean)
            return getattr(self, '_m_solar_cell_minus_z', None)

        @property
        def v3_3_2_status(self):
            if hasattr(self, '_m_v3_3_2_status'):
                return self._m_v3_3_2_status

            self._m_v3_3_2_status = int(self.v3_3_2_status_boolean)
            return getattr(self, '_m_v3_3_2_status', None)

        @property
        def aprs_payload_1(self):
            if hasattr(self, '_m_aprs_payload_1'):
                return self._m_aprs_payload_1

            self._m_aprs_payload_1 = int(self.aprs_payload_1_boolean)
            return getattr(self, '_m_aprs_payload_1', None)


    class GmskOrDigi(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            _on = self.message_type2
            if _on == 4043305215:
                self.id3 = Dragonfly.Gmsk(self._io, self, self._root)
            else:
                self.id3 = Dragonfly.Digi(self._io, self, self._root)

        @property
        def message_type2(self):
            if hasattr(self, '_m_message_type2'):
                return self._m_message_type2

            _pos = self._io.pos()
            self._io.seek(15)
            self._m_message_type2 = self._io.read_u4be()
            self._io.seek(_pos)
            return getattr(self, '_m_message_type2', None)


    class Cw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.cw_name_callsign = (self._io.read_bytes(15)).decode(u"ASCII")
            if not self.cw_name_callsign == u"dragonflyjg6yow":
                raise kaitaistruct.ValidationNotEqualError(u"dragonflyjg6yow", self.cw_name_callsign, self._io, u"/types/cw/seq/0")
            self.bat_v_raw = self._io.read_u1()
            self.bat_i_raw = self._io.read_u1()
            self.bat_t_raw = self._io.read_u1()
            self.value1 = self._io.read_u1()
            self.value2 = self._io.read_u1()
            self.value3 = self._io.read_u1()
            self.value4 = self._io.read_u1()
            self.time_after_last_reset = self._io.read_u1()
            self.lengthcheck = (self._io.read_bytes_full()).decode(u"utf-8")

        @property
        def ant_1_set_count(self):
            if hasattr(self, '_m_ant_1_set_count'):
                return self._m_ant_1_set_count

            self._m_ant_1_set_count = ((self.value2 & 112) >> 4)
            return getattr(self, '_m_ant_1_set_count', None)

        @property
        def bat_v(self):
            if hasattr(self, '_m_bat_v'):
                return self._m_bat_v

            self._m_bat_v = ((6.512 * self.bat_v_raw) / 256)
            return getattr(self, '_m_bat_v', None)

        @property
        def main_pic_power_line_status(self):
            if hasattr(self, '_m_main_pic_power_line_status'):
                return self._m_main_pic_power_line_status

            self._m_main_pic_power_line_status = ((self.value4 & 128) >> 7)
            return getattr(self, '_m_main_pic_power_line_status', None)

        @property
        def value3_hex_right_digit(self):
            if hasattr(self, '_m_value3_hex_right_digit'):
                return self._m_value3_hex_right_digit

            self._m_value3_hex_right_digit = (u"a" if str(self.value3_hex_right) == u"10" else (u"b" if str(self.value3_hex_right) == u"11" else (u"c" if str(self.value3_hex_right) == u"12" else (u"d" if str(self.value3_hex_right) == u"13" else (u"e" if str(self.value3_hex_right) == u"14" else (u"f" if str(self.value3_hex_right) == u"15" else str(self.value3_hex_right)))))))
            return getattr(self, '_m_value3_hex_right_digit', None)

        @property
        def necessary_for_lengthcheck(self):
            if hasattr(self, '_m_necessary_for_lengthcheck'):
                return self._m_necessary_for_lengthcheck

            if len(self.lengthcheck) != 0:
                self._m_necessary_for_lengthcheck = int(self.lengthcheck) // 0

            return getattr(self, '_m_necessary_for_lengthcheck', None)

        @property
        def aprs_payload_3(self):
            if hasattr(self, '_m_aprs_payload_3'):
                return self._m_aprs_payload_3

            self._m_aprs_payload_3 = ((self.value3 & 8) >> 3)
            return getattr(self, '_m_aprs_payload_3', None)

        @property
        def value1_hex_left(self):
            if hasattr(self, '_m_value1_hex_left'):
                return self._m_value1_hex_left

            self._m_value1_hex_left = self.value1 // 16
            return getattr(self, '_m_value1_hex_left', None)

        @property
        def unreg1_status(self):
            if hasattr(self, '_m_unreg1_status'):
                return self._m_unreg1_status

            self._m_unreg1_status = ((self.value4 & 4) >> 2)
            return getattr(self, '_m_unreg1_status', None)

        @property
        def kill_main(self):
            if hasattr(self, '_m_kill_main'):
                return self._m_kill_main

            self._m_kill_main = ((self.value1 & 128) >> 7)
            return getattr(self, '_m_kill_main', None)

        @property
        def bat_t_raw_hex_right(self):
            if hasattr(self, '_m_bat_t_raw_hex_right'):
                return self._m_bat_t_raw_hex_right

            self._m_bat_t_raw_hex_right = (self.bat_t_raw % 16)
            return getattr(self, '_m_bat_t_raw_hex_right', None)

        @property
        def unreg2_status(self):
            if hasattr(self, '_m_unreg2_status'):
                return self._m_unreg2_status

            self._m_unreg2_status = ((self.value4 & 2) >> 1)
            return getattr(self, '_m_unreg2_status', None)

        @property
        def time_after_last_reset_hex_right_digit(self):
            if hasattr(self, '_m_time_after_last_reset_hex_right_digit'):
                return self._m_time_after_last_reset_hex_right_digit

            self._m_time_after_last_reset_hex_right_digit = (u"a" if str(self.time_after_last_reset_hex_right) == u"10" else (u"b" if str(self.time_after_last_reset_hex_right) == u"11" else (u"c" if str(self.time_after_last_reset_hex_right) == u"12" else (u"d" if str(self.time_after_last_reset_hex_right) == u"13" else (u"e" if str(self.time_after_last_reset_hex_right) == u"14" else (u"f" if str(self.time_after_last_reset_hex_right) == u"15" else str(self.time_after_last_reset_hex_right)))))))
            return getattr(self, '_m_time_after_last_reset_hex_right_digit', None)

        @property
        def bat_i_raw_hex_right(self):
            if hasattr(self, '_m_bat_i_raw_hex_right'):
                return self._m_bat_i_raw_hex_right

            self._m_bat_i_raw_hex_right = (self.bat_i_raw % 16)
            return getattr(self, '_m_bat_i_raw_hex_right', None)

        @property
        def value2_hex_right(self):
            if hasattr(self, '_m_value2_hex_right'):
                return self._m_value2_hex_right

            self._m_value2_hex_right = (self.value2 % 16)
            return getattr(self, '_m_value2_hex_right', None)

        @property
        def value3_hex(self):
            if hasattr(self, '_m_value3_hex'):
                return self._m_value3_hex

            self._m_value3_hex = (u".." if self.value3_hex_left_digit + self.value3_hex_right_digit == u"ff" else self.value3_hex_left_digit + self.value3_hex_right_digit)
            return getattr(self, '_m_value3_hex', None)

        @property
        def bat_v_raw_hex(self):
            if hasattr(self, '_m_bat_v_raw_hex'):
                return self._m_bat_v_raw_hex

            self._m_bat_v_raw_hex = (u".." if self.bat_v_raw_hex_left_digit + self.bat_v_raw_hex_right_digit == u"ff" else self.bat_v_raw_hex_left_digit + self.bat_v_raw_hex_right_digit)
            return getattr(self, '_m_bat_v_raw_hex', None)

        @property
        def value3_hex_right(self):
            if hasattr(self, '_m_value3_hex_right'):
                return self._m_value3_hex_right

            self._m_value3_hex_right = (self.value3 % 16)
            return getattr(self, '_m_value3_hex_right', None)

        @property
        def solar_cell_plus_y(self):
            if hasattr(self, '_m_solar_cell_plus_y'):
                return self._m_solar_cell_plus_y

            self._m_solar_cell_plus_y = ((self.value1 & 16) >> 4)
            return getattr(self, '_m_solar_cell_plus_y', None)

        @property
        def ant_2_set_count(self):
            if hasattr(self, '_m_ant_2_set_count'):
                return self._m_ant_2_set_count

            self._m_ant_2_set_count = (self.value2 & 7)
            return getattr(self, '_m_ant_2_set_count', None)

        @property
        def aprs_reference_1(self):
            if hasattr(self, '_m_aprs_reference_1'):
                return self._m_aprs_reference_1

            self._m_aprs_reference_1 = ((self.value3 & 128) >> 7)
            return getattr(self, '_m_aprs_reference_1', None)

        @property
        def value1_hex_right_digit(self):
            if hasattr(self, '_m_value1_hex_right_digit'):
                return self._m_value1_hex_right_digit

            self._m_value1_hex_right_digit = (u"a" if str(self.value1_hex_right) == u"10" else (u"b" if str(self.value1_hex_right) == u"11" else (u"c" if str(self.value1_hex_right) == u"12" else (u"d" if str(self.value1_hex_right) == u"13" else (u"e" if str(self.value1_hex_right) == u"14" else (u"f" if str(self.value1_hex_right) == u"15" else str(self.value1_hex_right)))))))
            return getattr(self, '_m_value1_hex_right_digit', None)

        @property
        def aprs_payload_5(self):
            if hasattr(self, '_m_aprs_payload_5'):
                return self._m_aprs_payload_5

            self._m_aprs_payload_5 = ((self.value3 & 2) >> 1)
            return getattr(self, '_m_aprs_payload_5', None)

        @property
        def time_after_last_reset_hex_left_digit(self):
            if hasattr(self, '_m_time_after_last_reset_hex_left_digit'):
                return self._m_time_after_last_reset_hex_left_digit

            self._m_time_after_last_reset_hex_left_digit = (u"a" if str(self.time_after_last_reset_hex_left) == u"10" else (u"b" if str(self.time_after_last_reset_hex_left) == u"11" else (u"c" if str(self.time_after_last_reset_hex_left) == u"12" else (u"d" if str(self.time_after_last_reset_hex_left) == u"13" else (u"e" if str(self.time_after_last_reset_hex_left) == u"14" else (u"f" if str(self.time_after_last_reset_hex_left) == u"15" else str(self.time_after_last_reset_hex_left)))))))
            return getattr(self, '_m_time_after_last_reset_hex_left_digit', None)

        @property
        def bat_i_raw_hex_right_digit(self):
            if hasattr(self, '_m_bat_i_raw_hex_right_digit'):
                return self._m_bat_i_raw_hex_right_digit

            self._m_bat_i_raw_hex_right_digit = (u"a" if str(self.bat_i_raw_hex_right) == u"10" else (u"b" if str(self.bat_i_raw_hex_right) == u"11" else (u"c" if str(self.bat_i_raw_hex_right) == u"12" else (u"d" if str(self.bat_i_raw_hex_right) == u"13" else (u"e" if str(self.bat_i_raw_hex_right) == u"14" else (u"f" if str(self.bat_i_raw_hex_right) == u"15" else str(self.bat_i_raw_hex_right)))))))
            return getattr(self, '_m_bat_i_raw_hex_right_digit', None)

        @property
        def ant_2_deploy(self):
            if hasattr(self, '_m_ant_2_deploy'):
                return self._m_ant_2_deploy

            self._m_ant_2_deploy = ((self.value2 & 8) >> 3)
            return getattr(self, '_m_ant_2_deploy', None)

        @property
        def aprs_payload_2(self):
            if hasattr(self, '_m_aprs_payload_2'):
                return self._m_aprs_payload_2

            self._m_aprs_payload_2 = ((self.value3 & 16) >> 4)
            return getattr(self, '_m_aprs_payload_2', None)

        @property
        def time_after_last_reset_hex(self):
            if hasattr(self, '_m_time_after_last_reset_hex'):
                return self._m_time_after_last_reset_hex

            self._m_time_after_last_reset_hex = (u".." if self.time_after_last_reset_hex_left_digit + self.time_after_last_reset_hex_right_digit == u"ff" else self.time_after_last_reset_hex_left_digit + self.time_after_last_reset_hex_right_digit)
            return getattr(self, '_m_time_after_last_reset_hex', None)

        @property
        def v5_status(self):
            if hasattr(self, '_m_v5_status'):
                return self._m_v5_status

            self._m_v5_status = ((self.value4 & 8) >> 3)
            return getattr(self, '_m_v5_status', None)

        @property
        def com_pic_power_line_status(self):
            if hasattr(self, '_m_com_pic_power_line_status'):
                return self._m_com_pic_power_line_status

            self._m_com_pic_power_line_status = ((self.value4 & 64) >> 6)
            return getattr(self, '_m_com_pic_power_line_status', None)

        @property
        def beacon_type(self):
            if hasattr(self, '_m_beacon_type'):
                return self._m_beacon_type

            self._m_beacon_type = u"CW"
            return getattr(self, '_m_beacon_type', None)

        @property
        def value3_hex_left(self):
            if hasattr(self, '_m_value3_hex_left'):
                return self._m_value3_hex_left

            self._m_value3_hex_left = self.value3 // 16
            return getattr(self, '_m_value3_hex_left', None)

        @property
        def solar_cell_plus_z(self):
            if hasattr(self, '_m_solar_cell_plus_z'):
                return self._m_solar_cell_plus_z

            self._m_solar_cell_plus_z = ((self.value1 & 8) >> 3)
            return getattr(self, '_m_solar_cell_plus_z', None)

        @property
        def solar_cell_minus_x(self):
            if hasattr(self, '_m_solar_cell_minus_x'):
                return self._m_solar_cell_minus_x

            self._m_solar_cell_minus_x = ((self.value1 & 4) >> 2)
            return getattr(self, '_m_solar_cell_minus_x', None)

        @property
        def bat_i_raw_hex_left_digit(self):
            if hasattr(self, '_m_bat_i_raw_hex_left_digit'):
                return self._m_bat_i_raw_hex_left_digit

            self._m_bat_i_raw_hex_left_digit = (u"a" if str(self.bat_i_raw_hex_left) == u"10" else (u"b" if str(self.bat_i_raw_hex_left) == u"11" else (u"c" if str(self.bat_i_raw_hex_left) == u"12" else (u"d" if str(self.bat_i_raw_hex_left) == u"13" else (u"e" if str(self.bat_i_raw_hex_left) == u"14" else (u"f" if str(self.bat_i_raw_hex_left) == u"15" else str(self.bat_i_raw_hex_left)))))))
            return getattr(self, '_m_bat_i_raw_hex_left_digit', None)

        @property
        def bat_i(self):
            if hasattr(self, '_m_bat_i'):
                return self._m_bat_i

            self._m_bat_i = ((-2.99589 * self.bat_i_raw) + 6129.78533)
            return getattr(self, '_m_bat_i', None)

        @property
        def bat_v_raw_hex_right(self):
            if hasattr(self, '_m_bat_v_raw_hex_right'):
                return self._m_bat_v_raw_hex_right

            self._m_bat_v_raw_hex_right = (self.bat_v_raw % 16)
            return getattr(self, '_m_bat_v_raw_hex_right', None)

        @property
        def aprs_reference_2(self):
            if hasattr(self, '_m_aprs_reference_2'):
                return self._m_aprs_reference_2

            self._m_aprs_reference_2 = ((self.value3 & 64) >> 6)
            return getattr(self, '_m_aprs_reference_2', None)

        @property
        def bat_t_raw_hex_left(self):
            if hasattr(self, '_m_bat_t_raw_hex_left'):
                return self._m_bat_t_raw_hex_left

            self._m_bat_t_raw_hex_left = self.bat_t_raw // 16
            return getattr(self, '_m_bat_t_raw_hex_left', None)

        @property
        def ant_1_deploy(self):
            if hasattr(self, '_m_ant_1_deploy'):
                return self._m_ant_1_deploy

            self._m_ant_1_deploy = ((self.value2 & 128) >> 7)
            return getattr(self, '_m_ant_1_deploy', None)

        @property
        def bat_t_raw_hex_left_digit(self):
            if hasattr(self, '_m_bat_t_raw_hex_left_digit'):
                return self._m_bat_t_raw_hex_left_digit

            self._m_bat_t_raw_hex_left_digit = (u"a" if str(self.bat_t_raw_hex_left) == u"10" else (u"b" if str(self.bat_t_raw_hex_left) == u"11" else (u"c" if str(self.bat_t_raw_hex_left) == u"12" else (u"d" if str(self.bat_t_raw_hex_left) == u"13" else (u"e" if str(self.bat_t_raw_hex_left) == u"14" else (u"f" if str(self.bat_t_raw_hex_left) == u"15" else str(self.bat_t_raw_hex_left)))))))
            return getattr(self, '_m_bat_t_raw_hex_left_digit', None)

        @property
        def value4_hex_left_digit(self):
            if hasattr(self, '_m_value4_hex_left_digit'):
                return self._m_value4_hex_left_digit

            self._m_value4_hex_left_digit = (u"a" if str(self.value4_hex_left) == u"10" else (u"b" if str(self.value4_hex_left) == u"11" else (u"c" if str(self.value4_hex_left) == u"12" else (u"d" if str(self.value4_hex_left) == u"13" else (u"e" if str(self.value4_hex_left) == u"14" else (u"f" if str(self.value4_hex_left) == u"15" else str(self.value4_hex_left)))))))
            return getattr(self, '_m_value4_hex_left_digit', None)

        @property
        def value1_hex_right(self):
            if hasattr(self, '_m_value1_hex_right'):
                return self._m_value1_hex_right

            self._m_value1_hex_right = (self.value1 % 16)
            return getattr(self, '_m_value1_hex_right', None)

        @property
        def value4_hex_right(self):
            if hasattr(self, '_m_value4_hex_right'):
                return self._m_value4_hex_right

            self._m_value4_hex_right = (self.value4 % 16)
            return getattr(self, '_m_value4_hex_right', None)

        @property
        def time_after_last_reset_hex_right(self):
            if hasattr(self, '_m_time_after_last_reset_hex_right'):
                return self._m_time_after_last_reset_hex_right

            self._m_time_after_last_reset_hex_right = (self.time_after_last_reset % 16)
            return getattr(self, '_m_time_after_last_reset_hex_right', None)

        @property
        def value4_hex_right_digit(self):
            if hasattr(self, '_m_value4_hex_right_digit'):
                return self._m_value4_hex_right_digit

            self._m_value4_hex_right_digit = (u"a" if str(self.value4_hex_right) == u"10" else (u"b" if str(self.value4_hex_right) == u"11" else (u"c" if str(self.value4_hex_right) == u"12" else (u"d" if str(self.value4_hex_right) == u"13" else (u"e" if str(self.value4_hex_right) == u"14" else (u"f" if str(self.value4_hex_right) == u"15" else str(self.value4_hex_right)))))))
            return getattr(self, '_m_value4_hex_right_digit', None)

        @property
        def value1_hex(self):
            if hasattr(self, '_m_value1_hex'):
                return self._m_value1_hex

            self._m_value1_hex = (u".." if self.value1_hex_left_digit + self.value1_hex_right_digit == u"ff" else self.value1_hex_left_digit + self.value1_hex_right_digit)
            return getattr(self, '_m_value1_hex', None)

        @property
        def bat_v_raw_hex_right_digit(self):
            if hasattr(self, '_m_bat_v_raw_hex_right_digit'):
                return self._m_bat_v_raw_hex_right_digit

            self._m_bat_v_raw_hex_right_digit = (u"a" if str(self.bat_v_raw_hex_right) == u"10" else (u"b" if str(self.bat_v_raw_hex_right) == u"11" else (u"c" if str(self.bat_v_raw_hex_right) == u"12" else (u"d" if str(self.bat_v_raw_hex_right) == u"13" else (u"e" if str(self.bat_v_raw_hex_right) == u"14" else (u"f" if str(self.bat_v_raw_hex_right) == u"15" else str(self.bat_v_raw_hex_right)))))))
            return getattr(self, '_m_bat_v_raw_hex_right_digit', None)

        @property
        def bat_v_raw_hex_left(self):
            if hasattr(self, '_m_bat_v_raw_hex_left'):
                return self._m_bat_v_raw_hex_left

            self._m_bat_v_raw_hex_left = self.bat_v_raw // 16
            return getattr(self, '_m_bat_v_raw_hex_left', None)

        @property
        def cw_beacon(self):
            if hasattr(self, '_m_cw_beacon'):
                return self._m_cw_beacon

            self._m_cw_beacon = self.bat_v_raw_hex + self.bat_i_raw_hex + self.bat_t_raw_hex + self.value1_hex + self.value2_hex + self.value3_hex + self.value4_hex + self.time_after_last_reset_hex
            return getattr(self, '_m_cw_beacon', None)

        @property
        def v3_3_1_status(self):
            if hasattr(self, '_m_v3_3_1_status'):
                return self._m_v3_3_1_status

            self._m_v3_3_1_status = ((self.value4 & 32) >> 5)
            return getattr(self, '_m_v3_3_1_status', None)

        @property
        def value2_hex_right_digit(self):
            if hasattr(self, '_m_value2_hex_right_digit'):
                return self._m_value2_hex_right_digit

            self._m_value2_hex_right_digit = (u"a" if str(self.value2_hex_right) == u"10" else (u"b" if str(self.value2_hex_right) == u"11" else (u"c" if str(self.value2_hex_right) == u"12" else (u"d" if str(self.value2_hex_right) == u"13" else (u"e" if str(self.value2_hex_right) == u"14" else (u"f" if str(self.value2_hex_right) == u"15" else str(self.value2_hex_right)))))))
            return getattr(self, '_m_value2_hex_right_digit', None)

        @property
        def value4_hex_left(self):
            if hasattr(self, '_m_value4_hex_left'):
                return self._m_value4_hex_left

            self._m_value4_hex_left = self.value4 // 16
            return getattr(self, '_m_value4_hex_left', None)

        @property
        def value2_hex_left_digit(self):
            if hasattr(self, '_m_value2_hex_left_digit'):
                return self._m_value2_hex_left_digit

            self._m_value2_hex_left_digit = (u"a" if str(self.value2_hex_left) == u"10" else (u"b" if str(self.value2_hex_left) == u"11" else (u"c" if str(self.value2_hex_left) == u"12" else (u"d" if str(self.value2_hex_left) == u"13" else (u"e" if str(self.value2_hex_left) == u"14" else (u"f" if str(self.value2_hex_left) == u"15" else str(self.value2_hex_left)))))))
            return getattr(self, '_m_value2_hex_left_digit', None)

        @property
        def bat_t_raw_hex(self):
            if hasattr(self, '_m_bat_t_raw_hex'):
                return self._m_bat_t_raw_hex

            self._m_bat_t_raw_hex = (u".." if self.bat_t_raw_hex_left_digit + self.bat_t_raw_hex_right_digit == u"ff" else self.bat_t_raw_hex_left_digit + self.bat_t_raw_hex_right_digit)
            return getattr(self, '_m_bat_t_raw_hex', None)

        @property
        def solar_cell_plus_x(self):
            if hasattr(self, '_m_solar_cell_plus_x'):
                return self._m_solar_cell_plus_x

            self._m_solar_cell_plus_x = ((self.value1 & 32) >> 5)
            return getattr(self, '_m_solar_cell_plus_x', None)

        @property
        def value2_hex(self):
            if hasattr(self, '_m_value2_hex'):
                return self._m_value2_hex

            self._m_value2_hex = (u".." if self.value2_hex_left_digit + self.value2_hex_right_digit == u"ff" else self.value2_hex_left_digit + self.value2_hex_right_digit)
            return getattr(self, '_m_value2_hex', None)

        @property
        def kill_fab(self):
            if hasattr(self, '_m_kill_fab'):
                return self._m_kill_fab

            self._m_kill_fab = ((self.value1 & 64) >> 6)
            return getattr(self, '_m_kill_fab', None)

        @property
        def value4_hex(self):
            if hasattr(self, '_m_value4_hex'):
                return self._m_value4_hex

            self._m_value4_hex = (u".." if self.value4_hex_left_digit + self.value4_hex_right_digit == u"ff" else self.value4_hex_left_digit + self.value4_hex_right_digit)
            return getattr(self, '_m_value4_hex', None)

        @property
        def bat_t(self):
            if hasattr(self, '_m_bat_t'):
                return self._m_bat_t

            self._m_bat_t = (75 - ((self.bat_t_raw * 97.68) / 256))
            return getattr(self, '_m_bat_t', None)

        @property
        def bat_v_raw_hex_left_digit(self):
            if hasattr(self, '_m_bat_v_raw_hex_left_digit'):
                return self._m_bat_v_raw_hex_left_digit

            self._m_bat_v_raw_hex_left_digit = (u"a" if str(self.bat_v_raw_hex_left) == u"10" else (u"b" if str(self.bat_v_raw_hex_left) == u"11" else (u"c" if str(self.bat_v_raw_hex_left) == u"12" else (u"d" if str(self.bat_v_raw_hex_left) == u"13" else (u"e" if str(self.bat_v_raw_hex_left) == u"14" else (u"f" if str(self.bat_v_raw_hex_left) == u"15" else str(self.bat_v_raw_hex_left)))))))
            return getattr(self, '_m_bat_v_raw_hex_left_digit', None)

        @property
        def solar_cell_minus_y(self):
            if hasattr(self, '_m_solar_cell_minus_y'):
                return self._m_solar_cell_minus_y

            self._m_solar_cell_minus_y = ((self.value1 & 2) >> 1)
            return getattr(self, '_m_solar_cell_minus_y', None)

        @property
        def value3_hex_left_digit(self):
            if hasattr(self, '_m_value3_hex_left_digit'):
                return self._m_value3_hex_left_digit

            self._m_value3_hex_left_digit = (u"a" if str(self.value3_hex_left) == u"10" else (u"b" if str(self.value3_hex_left) == u"11" else (u"c" if str(self.value3_hex_left) == u"12" else (u"d" if str(self.value3_hex_left) == u"13" else (u"e" if str(self.value3_hex_left) == u"14" else (u"f" if str(self.value3_hex_left) == u"15" else str(self.value3_hex_left)))))))
            return getattr(self, '_m_value3_hex_left_digit', None)

        @property
        def bat_i_raw_hex_left(self):
            if hasattr(self, '_m_bat_i_raw_hex_left'):
                return self._m_bat_i_raw_hex_left

            self._m_bat_i_raw_hex_left = self.bat_i_raw // 16
            return getattr(self, '_m_bat_i_raw_hex_left', None)

        @property
        def aprs_payload_4(self):
            if hasattr(self, '_m_aprs_payload_4'):
                return self._m_aprs_payload_4

            self._m_aprs_payload_4 = ((self.value3 & 4) >> 2)
            return getattr(self, '_m_aprs_payload_4', None)

        @property
        def solar_cell_minus_z(self):
            if hasattr(self, '_m_solar_cell_minus_z'):
                return self._m_solar_cell_minus_z

            self._m_solar_cell_minus_z = (self.value1 & 1)
            return getattr(self, '_m_solar_cell_minus_z', None)

        @property
        def value2_hex_left(self):
            if hasattr(self, '_m_value2_hex_left'):
                return self._m_value2_hex_left

            self._m_value2_hex_left = self.value2 // 16
            return getattr(self, '_m_value2_hex_left', None)

        @property
        def v3_3_2_status(self):
            if hasattr(self, '_m_v3_3_2_status'):
                return self._m_v3_3_2_status

            self._m_v3_3_2_status = ((self.value4 & 16) >> 4)
            return getattr(self, '_m_v3_3_2_status', None)

        @property
        def bat_t_raw_hex_right_digit(self):
            if hasattr(self, '_m_bat_t_raw_hex_right_digit'):
                return self._m_bat_t_raw_hex_right_digit

            self._m_bat_t_raw_hex_right_digit = (u"a" if str(self.bat_t_raw_hex_right) == u"10" else (u"b" if str(self.bat_t_raw_hex_right) == u"11" else (u"c" if str(self.bat_t_raw_hex_right) == u"12" else (u"d" if str(self.bat_t_raw_hex_right) == u"13" else (u"e" if str(self.bat_t_raw_hex_right) == u"14" else (u"f" if str(self.bat_t_raw_hex_right) == u"15" else str(self.bat_t_raw_hex_right)))))))
            return getattr(self, '_m_bat_t_raw_hex_right_digit', None)

        @property
        def aprs_payload_1(self):
            if hasattr(self, '_m_aprs_payload_1'):
                return self._m_aprs_payload_1

            self._m_aprs_payload_1 = ((self.value3 & 32) >> 5)
            return getattr(self, '_m_aprs_payload_1', None)

        @property
        def value1_hex_left_digit(self):
            if hasattr(self, '_m_value1_hex_left_digit'):
                return self._m_value1_hex_left_digit

            self._m_value1_hex_left_digit = (u"a" if str(self.value1_hex_left) == u"10" else (u"b" if str(self.value1_hex_left) == u"11" else (u"c" if str(self.value1_hex_left) == u"12" else (u"d" if str(self.value1_hex_left) == u"13" else (u"e" if str(self.value1_hex_left) == u"14" else (u"f" if str(self.value1_hex_left) == u"15" else str(self.value1_hex_left)))))))
            return getattr(self, '_m_value1_hex_left_digit', None)

        @property
        def bat_i_raw_hex(self):
            if hasattr(self, '_m_bat_i_raw_hex'):
                return self._m_bat_i_raw_hex

            self._m_bat_i_raw_hex = (u".." if self.bat_i_raw_hex_left_digit + self.bat_i_raw_hex_right_digit == u"ff" else self.bat_i_raw_hex_left_digit + self.bat_i_raw_hex_right_digit)
            return getattr(self, '_m_bat_i_raw_hex', None)

        @property
        def time_after_last_reset_hex_left(self):
            if hasattr(self, '_m_time_after_last_reset_hex_left'):
                return self._m_time_after_last_reset_hex_left

            self._m_time_after_last_reset_hex_left = self.time_after_last_reset // 16
            return getattr(self, '_m_time_after_last_reset_hex_left', None)


    class Digi(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_frame = Dragonfly.Digi.Ax25Frame(self._io, self, self._root)

        class Ax25Frame(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.ax25_header = Dragonfly.Digi.Ax25Header(self._io, self, self._root)
                self._raw_ax25_info = self._io.read_bytes_full()
                _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
                self.ax25_info = Dragonfly.Digi.Ax25InfoData(_io__raw_ax25_info, self, self._root)


        class Ax25Header(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.dest_callsign_raw = Dragonfly.Digi.CallsignRaw(self._io, self, self._root)
                self.dest_ssid_raw = Dragonfly.Digi.SsidMask(self._io, self, self._root)
                self.src_callsign_raw = Dragonfly.Digi.CallsignRaw(self._io, self, self._root)
                self.src_ssid_raw = Dragonfly.Digi.SsidMask(self._io, self, self._root)
                if (self.src_ssid_raw.ssid_mask & 1) == 0:
                    self.repeater = Dragonfly.Digi.Repeater(self._io, self, self._root)

                self.ctl = self._io.read_u1()
                self.pid = self._io.read_u1()


        class Callsign(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


        class SsidMask(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.ssid_mask = self._io.read_u1()

            @property
            def ssid(self):
                if hasattr(self, '_m_ssid'):
                    return self._m_ssid

                self._m_ssid = ((self.ssid_mask & 31) >> 1)
                return getattr(self, '_m_ssid', None)

            @property
            def hbit(self):
                if hasattr(self, '_m_hbit'):
                    return self._m_hbit

                self._m_hbit = ((self.ssid_mask & 128) >> 7)
                return getattr(self, '_m_hbit', None)


        class Repeaters(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.rpt_callsign_raw = Dragonfly.Digi.CallsignRaw(self._io, self, self._root)
                self.rpt_ssid_raw = Dragonfly.Digi.SsidMask(self._io, self, self._root)


        class Repeater(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.rpt_instance = []
                i = 0
                while True:
                    _ = Dragonfly.Digi.Repeaters(self._io, self, self._root)
                    self.rpt_instance.append(_)
                    if (_.rpt_ssid_raw.ssid_mask & 1) == 1:
                        break
                    i += 1


        class CallsignRaw(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self._raw__raw_callsign_ror = self._io.read_bytes(6)
                self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
                _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
                self.callsign_ror = Dragonfly.Digi.Callsign(_io__raw_callsign_ror, self, self._root)


        class Ax25InfoData(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.digi_message = (self._io.read_bytes_full()).decode(u"utf-8")







# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO
import satnogsdecoders.process


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Geoscanedelveis(KaitaiStruct):
    """:field dest_callsign: data.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field src_callsign: data.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid: data.ax25_header.src_ssid_raw.ssid
    :field dest_ssid: data.ax25_header.dest_ssid_raw.ssid
    :field ctl: data.ax25_header.ctl
    :field pid: data.ax25_header.pid
    :field obc_timestamp: data.payload.obc_timestamp
    :field eps_cell_current: data.payload.eps_cell_current
    :field eps_system_current: data.payload.eps_system_current
    :field eps_cell_voltage_half: data.payload.eps_cell_voltage_half
    :field eps_cell_voltage_full: data.payload.eps_cell_voltage_full
    :field adc_temperature_pos_x: data.payload.adc_temperature_pos_x
    :field adc_temperature_neg_x: data.payload.adc_temperature_neg_x
    :field adc_temperature_pos_y: data.payload.adc_temperature_pos_y
    :field adc_temperature_neg_y: data.payload.adc_temperature_neg_y
    :field adc_temperature_pos_z: data.payload.adc_temperature_pos_z
    :field adc_temperature_neg_z: data.payload.adc_temperature_neg_z
    :field adc_temperature_cell1: data.payload.adc_temperature_cell1
    :field adc_temperature_cell2: data.payload.adc_temperature_cell2
    :field obc_cpu_load: data.payload.obc_cpu_load
    :field obc_boot_count: data.payload.obc_boot_count
    :field comm_boot_count: data.payload.comm_boot_count
    :field comm_rssi: data.payload.comm_rssi
    :field sat_id: data.geoscan_header.sat_id
    :field info: data.geoscan_header.info
    :field offset: data.geoscan_header.offset
    :field cmd: data.geoscan_data.cmd
    :field adc_timestamp: data.geoscan_data.payload.adc_timestamp
    :field sun_sensor_pos_x: data.geoscan_data.payload.sun_sensor_pos_x
    :field sun_sensor_neg_x: data.geoscan_data.payload.sun_sensor_neg_x
    :field sun_sensor_pos_y: data.geoscan_data.payload.sun_sensor_pos_y
    :field sun_sensor_neg_y: data.geoscan_data.payload.sun_sensor_neg_y
    :field sun_sensor_neg_z: data.geoscan_data.payload.sun_sensor_neg_z
    :field mag_sensor_pos_x: data.geoscan_data.payload.mag_sensor_pos_x
    :field mag_sensor_neg_x: data.geoscan_data.payload.mag_sensor_neg_x
    :field mag_sensor_pos_y: data.geoscan_data.payload.mag_sensor_pos_y
    :field mag_sensor_neg_y: data.geoscan_data.payload.mag_sensor_neg_y
    :field mag_sensor_pos_z: data.geoscan_data.payload.mag_sensor_pos_z
    :field mag_sensor_neg_z: data.geoscan_data.payload.mag_sensor_neg_z
    :field temperature_pos_x: data.geoscan_data.payload.temperature_pos_x
    :field temperature_neg_x: data.geoscan_data.payload.temperature_neg_x
    :field temperature_pos_y: data.geoscan_data.payload.temperature_pos_y
    :field temperature_neg_y: data.geoscan_data.payload.temperature_neg_y
    :field temperature_pos_z: data.geoscan_data.payload.temperature_pos_z
    :field temperature_neg_z: data.geoscan_data.payload.temperature_neg_z
    :field temperature_cell1: data.geoscan_data.payload.temperature_cell1
    :field temperature_cell2: data.geoscan_data.payload.temperature_cell2
    :field status: data.geoscan_data.payload.status
    :field eps_timestamp: data.geoscan_data.payload.eps_timestamp
    :field geoscan_data_eps_cell_current: data.geoscan_data.payload.eps_cell_current
    :field geoscan_data_eps_system_current: data.geoscan_data.payload.eps_system_current
    :field reserved0: data.geoscan_data.payload.reserved0
    :field reserved1: data.geoscan_data.payload.reserved1
    :field reserved2: data.geoscan_data.payload.reserved2
    :field reserved3: data.geoscan_data.payload.reserved3
    :field geoscan_data_eps_cell_voltage_half: data.geoscan_data.payload.eps_cell_voltage_half
    :field geoscan_data_eps_cell_voltage_full: data.geoscan_data.payload.eps_cell_voltage_full
    :field reserved4: data.geoscan_data.payload.reserved4
    :field reserved5: data.geoscan_data.payload.reserved5
    :field reserved6: data.geoscan_data.payload.reserved6
    :field gnss_timestamp: data.geoscan_data.payload.gnss_timestamp
    :field valid: data.geoscan_data.payload.valid
    :field year: data.geoscan_data.payload.year
    :field month: data.geoscan_data.payload.month
    :field day: data.geoscan_data.payload.day
    :field hour: data.geoscan_data.payload.hour
    :field minute: data.geoscan_data.payload.minute
    :field second: data.geoscan_data.payload.second
    :field dutc: data.geoscan_data.payload.dutc
    :field offset_gnss: data.geoscan_data.payload.offset_gnss
    :field x: data.geoscan_data.payload.x
    :field y: data.geoscan_data.payload.y
    :field z: data.geoscan_data.payload.z
    :field v_x: data.geoscan_data.payload.v_x
    :field v_y: data.geoscan_data.payload.v_y
    :field v_z: data.geoscan_data.payload.v_z
    :field vdop: data.geoscan_data.payload.vdop
    :field hdop: data.geoscan_data.payload.hdop
    :field pdop: data.geoscan_data.payload.pdop
    :field tdop: data.geoscan_data.payload.tdop
    :field fakel_timestamp: data.geoscan_data.payload.fakel_timestamp
    :field bitfield0: data.geoscan_data.payload.bitfield0
    :field bitfield1: data.geoscan_data.payload.bitfield1
    :field voltage_80v: data.geoscan_data.payload.voltage_80v
    :field voltage_13v: data.geoscan_data.payload.voltage_13v
    :field current_valve: data.geoscan_data.payload.current_valve
    :field voltage_valve: data.geoscan_data.payload.voltage_valve
    :field current_control_valve: data.geoscan_data.payload.current_control_valve
    :field voltage_control_valve: data.geoscan_data.payload.voltage_control_valve
    :field current_ek1: data.geoscan_data.payload.current_ek1
    :field voltage_ek1: data.geoscan_data.payload.voltage_ek1
    :field current_ek2: data.geoscan_data.payload.current_ek2
    :field voltage_ek2: data.geoscan_data.payload.voltage_ek2
    :field current_ek: data.geoscan_data.payload.current_ek
    :field voltage_ek: data.geoscan_data.payload.voltage_ek
    :field temperature_td1: data.geoscan_data.payload.temperature_td1
    :field reserved7: data.geoscan_data.payload.reserved7
    :field temperature_tp: data.geoscan_data.payload.temperature_tp
    :field pressure_cylinder: data.geoscan_data.payload.pressure_cylinder
    :field pressure_receiver: data.geoscan_data.payload.pressure_receiver
    :field switch_on_counter: data.geoscan_data.payload.switch_on_counter
    :field time_switched_on_counter: data.geoscan_data.payload.time_switched_on_counter
    :field bitfield3: data.geoscan_data.payload.bitfield3
    :field state: data.geoscan_data.payload.state
    :field reserved8: data.geoscan_data.payload.reserved8
    :field reserved9: data.geoscan_data.payload.reserved9
    :field pump_time: data.geoscan_data.payload.pump_time
    :field time_of_last_pump: data.geoscan_data.payload.time_of_last_pump
    :field reserved10: data.geoscan_data.payload.reserved10
    :field data_str: data.geoscan_data.payload.photo_data.data.data_str
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        _on = self.type
        if _on == 2223669894:
            self.data = Geoscanedelveis.Ax25Frame(self._io, self, self._root)
        elif _on == 16793089:
            self.data = Geoscanedelveis.GeoscanFrame(self._io, self, self._root)
        elif _on == 16793093:
            self.data = Geoscanedelveis.GeoscanFrame(self._io, self, self._root)

    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Geoscanedelveis.Ax25Header(self._io, self, self._root)
            self.payload = Geoscanedelveis.GeoscanBeaconTlm(self._io, self, self._root)


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Geoscanedelveis.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Geoscanedelveis.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Geoscanedelveis.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Geoscanedelveis.SsidMask(self._io, self, self._root)
            self.ctl = self._io.read_u1()
            self.pid = self._io.read_u1()


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")
            if not  ((self.callsign == u"BEACON") or (self.callsign == u"RS20S ")) :
                raise kaitaistruct.ValidationNotAnyOfError(self.callsign, self._io, u"/types/callsign/seq/0")


    class GeoscanFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.geoscan_header = Geoscanedelveis.GeoscanHeader(self._io, self, self._root)
            self.geoscan_data = Geoscanedelveis.GeoscanTlm(self._io, self, self._root)


    class StrB64T(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.data_str = (self._io.read_bytes_full()).decode(u"ASCII")


    class GeoscanPhoto(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw_photo_data = self._io.read_bytes_full()
            _io__raw_photo_data = KaitaiStream(BytesIO(self._raw_photo_data))
            self.photo_data = Geoscanedelveis.DataB64T(_io__raw_photo_data, self, self._root)


    class GeoscanHeader(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.sat_id = self._io.read_u1()
            self.info = self._io.read_u4le()
            self.offset = self._io.read_u2le()


    class DataB64T(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_data = self._io.read_bytes_full()
            _process = satnogsdecoders.process.B64encode()
            self._raw_data = _process.decode(self._raw__raw_data)
            _io__raw_data = KaitaiStream(BytesIO(self._raw_data))
            self.data = Geoscanedelveis.StrB64T(_io__raw_data, self, self._root)


    class GeoscanGnss(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.gnss_timestamp = self._io.read_u4le()
            self.valid = self._io.read_u1()
            if not  ((self.valid == 1)) :
                raise kaitaistruct.ValidationNotAnyOfError(self.valid, self._io, u"/types/geoscan_gnss/seq/1")
            self.year = self._io.read_u2le()
            self.month = self._io.read_u1()
            self.day = self._io.read_u1()
            self.hour = self._io.read_u1()
            self.minute = self._io.read_u1()
            self.second = self._io.read_u2le()
            self.dutc = self._io.read_u1()
            self.offset_gnss = self._io.read_u2le()
            self.x = self._io.read_s4le()
            self.y = self._io.read_s4le()
            self.z = self._io.read_s4le()
            self.v_x = self._io.read_s4le()
            self.v_y = self._io.read_s4le()
            self.v_z = self._io.read_s4le()
            self.vdop = self._io.read_u2le()
            self.hdop = self._io.read_u2le()
            self.pdop = self._io.read_u2le()
            self.tdop = self._io.read_u2le()


    class GeoscanEps(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.eps_timestamp = self._io.read_u4le()
            self.eps_cell_current = self._io.read_u4le()
            self.eps_system_current = self._io.read_u4le()
            self.reserved0 = self._io.read_u4le()
            self.reserved1 = self._io.read_u4le()
            self.reserved2 = self._io.read_s2le()
            self.reserved3 = self._io.read_u2le()
            self.eps_cell_voltage_half = self._io.read_u1()
            self.eps_cell_voltage_full = self._io.read_u1()
            self.reserved4 = self._io.read_s2le()
            self.reserved5 = self._io.read_s1()
            self.reserved6 = self._io.read_u4le()


    class GeoscanTlm(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.cmd = self._io.read_u1()
            _on = self.cmd
            if _on == 10:
                self.payload = Geoscanedelveis.GeoscanGnss(self._io, self, self._root)
            elif _on == 6:
                self.payload = Geoscanedelveis.GeoscanFakel(self._io, self, self._root)
            elif _on == 7:
                self.payload = Geoscanedelveis.GeoscanGnss(self._io, self, self._root)
            elif _on == 1:
                self.payload = Geoscanedelveis.GeoscanAdc(self._io, self, self._root)
            elif _on == 11:
                self.payload = Geoscanedelveis.GeoscanPhoto(self._io, self, self._root)
            elif _on == 5:
                self.payload = Geoscanedelveis.GeoscanFakel(self._io, self, self._root)
            elif _on == 8:
                self.payload = Geoscanedelveis.GeoscanGnss(self._io, self, self._root)
            elif _on == 9:
                self.payload = Geoscanedelveis.GeoscanGnss(self._io, self, self._root)
            elif _on == 2:
                self.payload = Geoscanedelveis.GeoscanEps(self._io, self, self._root)


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return getattr(self, '_m_ssid', None)


    class GeoscanBeaconTlm(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.obc_timestamp = self._io.read_u4le()
            self.eps_cell_current = self._io.read_u2le()
            self.eps_system_current = self._io.read_u2le()
            self.eps_cell_voltage_half = self._io.read_u2le()
            self.eps_cell_voltage_full = self._io.read_u2le()
            self.adc_temperature_pos_x = self._io.read_s1()
            self.adc_temperature_neg_x = self._io.read_s1()
            self.adc_temperature_pos_y = self._io.read_s1()
            self.adc_temperature_neg_y = self._io.read_s1()
            self.adc_temperature_pos_z = self._io.read_s1()
            self.adc_temperature_neg_z = self._io.read_s1()
            self.adc_temperature_cell1 = self._io.read_s1()
            self.adc_temperature_cell2 = self._io.read_s1()
            self.obc_cpu_load = self._io.read_u1()
            self.obc_boot_count = self._io.read_u2le()
            self.comm_boot_count = self._io.read_u2le()
            self.comm_rssi = self._io.read_s1()


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Geoscanedelveis.Callsign(_io__raw_callsign_ror, self, self._root)


    class GeoscanAdc(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.adc_timestamp = self._io.read_u4le()
            self.sun_sensor_pos_x = self._io.read_u4le()
            self.sun_sensor_neg_x = self._io.read_u4le()
            self.sun_sensor_pos_y = self._io.read_u4le()
            self.sun_sensor_neg_y = self._io.read_u4le()
            self.sun_sensor_neg_z = self._io.read_u4le()
            self.mag_sensor_pos_x = self._io.read_s1()
            self.mag_sensor_neg_x = self._io.read_s1()
            self.mag_sensor_pos_y = self._io.read_s1()
            self.mag_sensor_neg_y = self._io.read_s1()
            self.mag_sensor_pos_z = self._io.read_s1()
            self.mag_sensor_neg_z = self._io.read_s1()
            self.temperature_pos_x = self._io.read_s1()
            self.temperature_neg_x = self._io.read_s1()
            self.temperature_pos_y = self._io.read_s1()
            self.temperature_neg_y = self._io.read_s1()
            self.temperature_pos_z = self._io.read_s1()
            self.temperature_neg_z = self._io.read_s1()
            self.temperature_cell1 = self._io.read_s1()
            self.temperature_cell2 = self._io.read_s1()
            self.status = self._io.read_u4le()


    class GeoscanFakel(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.fakel_timestamp = self._io.read_u4le()
            self.bitfield0 = self._io.read_u1()
            self.bitfield1 = self._io.read_u1()
            self.voltage_80v = self._io.read_u2le()
            self.voltage_13v = self._io.read_u2le()
            self.current_valve = self._io.read_u2le()
            self.voltage_valve = self._io.read_u2le()
            self.current_control_valve = self._io.read_u2le()
            self.voltage_control_valve = self._io.read_u2le()
            self.current_ek1 = self._io.read_u2le()
            self.voltage_ek1 = self._io.read_u2le()
            self.current_ek2 = self._io.read_u2le()
            self.voltage_ek2 = self._io.read_u2le()
            self.current_ek = self._io.read_u2le()
            self.voltage_ek = self._io.read_u2le()
            self.temperature_td1 = self._io.read_u2le()
            self.reserved7 = self._io.read_u2le()
            self.temperature_tp = self._io.read_u2le()
            self.pressure_cylinder = self._io.read_u2le()
            self.pressure_receiver = self._io.read_u2le()
            self.switch_on_counter = self._io.read_u2le()
            self.time_switched_on_counter = self._io.read_u2le()
            self.bitfield3 = self._io.read_u1()
            self.state = self._io.read_u1()
            self.reserved8 = self._io.read_u1()
            self.reserved9 = self._io.read_u1()
            self.pump_time = self._io.read_u2le()
            self.time_of_last_pump = self._io.read_u2le()
            self.reserved10 = self._io.read_u1()


    @property
    def len(self):
        if hasattr(self, '_m_len'):
            return self._m_len

        self._m_len = self._io.size()
        return getattr(self, '_m_len', None)

    @property
    def type(self):
        if hasattr(self, '_m_type'):
            return self._m_type

        _pos = self._io.pos()
        self._io.seek(0)
        self._m_type = self._io.read_u4be()
        self._io.seek(_pos)
        return getattr(self, '_m_type', None)





# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Io86(KaitaiStruct):
    """:field telemetry_beacon: io86.type_check.ax25_frame.payload.ax25_info.telemetry_beacon
    :field counter: io86.type_check.ax25_frame.payload.ax25_info.counter
    :field analog1: io86.type_check.ax25_frame.payload.ax25_info.analog1
    :field analog2: io86.type_check.ax25_frame.payload.ax25_info.analog2
    :field analog3: io86.type_check.ax25_frame.payload.ax25_info.analog3
    :field analog4: io86.type_check.ax25_frame.payload.ax25_info.analog4
    :field analog5: io86.type_check.ax25_frame.payload.ax25_info.analog5
    :field digital1: io86.type_check.ax25_frame.payload.ax25_info.digital1
    :field digital2: io86.type_check.ax25_frame.payload.ax25_info.digital2
    :field digital3: io86.type_check.ax25_frame.payload.ax25_info.digital3
    :field digital4: io86.type_check.ax25_frame.payload.ax25_info.digital4
    :field digital5: io86.type_check.ax25_frame.payload.ax25_info.digital5
    :field digital6: io86.type_check.ax25_frame.payload.ax25_info.digital6
    :field digital7: io86.type_check.ax25_frame.payload.ax25_info.digital7
    :field digital8: io86.type_check.ax25_frame.payload.ax25_info.digital8
    :field dest_callsign: io86.type_check.ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field src_callsign: io86.type_check.ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid: io86.type_check.ax25_frame.ax25_header.src_ssid_raw.ssid
    :field dest_ssid: io86.type_check.ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field rpt_instance___callsign: io86.type_check.ax25_frame.ax25_header.repeater.rpt_instance.___.rpt_callsign_raw.callsign_ror.callsign
    :field rpt_instance___ssid: io86.type_check.ax25_frame.ax25_header.repeater.rpt_instance.___.rpt_ssid_raw.ssid
    :field rpt_instance___hbit: io86.type_check.ax25_frame.ax25_header.repeater.rpt_instance.___.rpt_ssid_raw.hbit
    :field ctl: io86.type_check.ax25_frame.ax25_header.ctl
    :field pid: io86.type_check.ax25_frame.payload.pid
    :field monitor: io86.type_check.ax25_frame.payload.ax25_info.data_monitor
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.io86 = Io86.Io86T(self._io, self, self._root)

    class Io86T(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            _on = self.check
            if _on == 21539:
                self.type_check = Io86.Telemetry(self._io, self, self._root)
            else:
                self.type_check = Io86.Digi(self._io, self, self._root)

        @property
        def check(self):
            if hasattr(self, '_m_check'):
                return self._m_check

            _pos = self._io.pos()
            self._io.seek(23)
            self._m_check = self._io.read_u2be()
            self._io.seek(_pos)
            return getattr(self, '_m_check', None)


    class Telemetry(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_frame = Io86.Telemetry.Ax25Frame(self._io, self, self._root)

        class Ax25Frame(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.ax25_header = Io86.Telemetry.Ax25Header(self._io, self, self._root)
                _on = (self.ax25_header.ctl & 19)
                if _on == 0:
                    self.payload = Io86.Telemetry.IFrame(self._io, self, self._root)
                elif _on == 3:
                    self.payload = Io86.Telemetry.UiFrame(self._io, self, self._root)
                elif _on == 19:
                    self.payload = Io86.Telemetry.UiFrame(self._io, self, self._root)
                elif _on == 16:
                    self.payload = Io86.Telemetry.IFrame(self._io, self, self._root)
                elif _on == 18:
                    self.payload = Io86.Telemetry.IFrame(self._io, self, self._root)
                elif _on == 2:
                    self.payload = Io86.Telemetry.IFrame(self._io, self, self._root)


        class Ax25Header(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.dest_callsign_raw = Io86.Telemetry.CallsignRaw(self._io, self, self._root)
                self.dest_ssid_raw = Io86.Telemetry.SsidMask(self._io, self, self._root)
                self.src_callsign_raw = Io86.Telemetry.CallsignRaw(self._io, self, self._root)
                self.src_ssid_raw = Io86.Telemetry.SsidMask(self._io, self, self._root)
                if (self.src_ssid_raw.ssid_mask & 1) == 0:
                    self.repeater = Io86.Telemetry.Repeater(self._io, self, self._root)

                self.ctl = self._io.read_u1()


        class UiFrame(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.pid = self._io.read_u1()
                self._raw_ax25_info = self._io.read_bytes_full()
                _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
                self.ax25_info = Io86.Telemetry.Ax25InfoData(_io__raw_ax25_info, self, self._root)


        class Callsign(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


        class IFrame(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.pid = self._io.read_u1()
                self._raw_ax25_info = self._io.read_bytes_full()
                _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
                self.ax25_info = Io86.Telemetry.Ax25InfoData(_io__raw_ax25_info, self, self._root)


        class SsidMask(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.ssid_mask = self._io.read_u1()

            @property
            def ssid(self):
                if hasattr(self, '_m_ssid'):
                    return self._m_ssid

                self._m_ssid = ((self.ssid_mask & 15) >> 1)
                return getattr(self, '_m_ssid', None)

            @property
            def hbit(self):
                if hasattr(self, '_m_hbit'):
                    return self._m_hbit

                self._m_hbit = ((self.ssid_mask & 128) >> 7)
                return getattr(self, '_m_hbit', None)


        class Repeaters(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.rpt_callsign_raw = Io86.Telemetry.CallsignRaw(self._io, self, self._root)
                self.rpt_ssid_raw = Io86.Telemetry.SsidMask(self._io, self, self._root)


        class Repeater(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.rpt_instance = []
                i = 0
                while True:
                    _ = Io86.Telemetry.Repeaters(self._io, self, self._root)
                    self.rpt_instance.append(_)
                    if (_.rpt_ssid_raw.ssid_mask & 1) == 1:
                        break
                    i += 1


        class CallsignRaw(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self._raw__raw_callsign_ror = self._io.read_bytes(6)
                self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
                _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
                self.callsign_ror = Io86.Telemetry.Callsign(_io__raw_callsign_ror, self, self._root)


        class Ax25InfoData(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.telemetry = (self._io.read_bytes(2)).decode(u"ASCII")
                if not self.telemetry == u"T#":
                    raise kaitaistruct.ValidationNotEqualError(u"T#", self.telemetry, self._io, u"/types/telemetry/types/ax25_info_data/seq/0")
                self.counter_ascii = (self._io.read_bytes_term(44, False, True, True)).decode(u"ASCII")
                self.analog1_ascii = (self._io.read_bytes_term(44, False, True, True)).decode(u"ASCII")
                self.analog2_ascii = (self._io.read_bytes_term(44, False, True, True)).decode(u"ASCII")
                self.analog3_ascii = (self._io.read_bytes_term(44, False, True, True)).decode(u"ASCII")
                self.analog4_ascii = (self._io.read_bytes_term(44, False, True, True)).decode(u"ASCII")
                self.analog5_ascii = (self._io.read_bytes_term(44, False, True, True)).decode(u"ASCII")
                self.digital1_ascii = (self._io.read_bytes(1)).decode(u"ASCII")
                self.digital2_ascii = (self._io.read_bytes(1)).decode(u"ASCII")
                self.digital3_ascii = (self._io.read_bytes(1)).decode(u"ASCII")
                self.digital4_ascii = (self._io.read_bytes(1)).decode(u"ASCII")
                self.digital5_ascii = (self._io.read_bytes(1)).decode(u"ASCII")
                self.digital6_ascii = (self._io.read_bytes(1)).decode(u"ASCII")
                self.digital7_ascii = (self._io.read_bytes(1)).decode(u"ASCII")
                self.digital8_ascii = (self._io.read_bytes(1)).decode(u"ASCII")

            @property
            def digital4(self):
                if hasattr(self, '_m_digital4'):
                    return self._m_digital4

                self._m_digital4 = int(self.digital4_ascii)
                return getattr(self, '_m_digital4', None)

            @property
            def counter(self):
                if hasattr(self, '_m_counter'):
                    return self._m_counter

                self._m_counter = int(self.counter_ascii)
                return getattr(self, '_m_counter', None)

            @property
            def analog4(self):
                if hasattr(self, '_m_analog4'):
                    return self._m_analog4

                self._m_analog4 = int(self.analog4_ascii)
                return getattr(self, '_m_analog4', None)

            @property
            def analog1(self):
                if hasattr(self, '_m_analog1'):
                    return self._m_analog1

                self._m_analog1 = int(self.analog1_ascii)
                return getattr(self, '_m_analog1', None)

            @property
            def digital5(self):
                if hasattr(self, '_m_digital5'):
                    return self._m_digital5

                self._m_digital5 = int(self.digital5_ascii)
                return getattr(self, '_m_digital5', None)

            @property
            def digital3(self):
                if hasattr(self, '_m_digital3'):
                    return self._m_digital3

                self._m_digital3 = int(self.digital3_ascii)
                return getattr(self, '_m_digital3', None)

            @property
            def digital6(self):
                if hasattr(self, '_m_digital6'):
                    return self._m_digital6

                self._m_digital6 = int(self.digital6_ascii)
                return getattr(self, '_m_digital6', None)

            @property
            def telemetry_beacon(self):
                if hasattr(self, '_m_telemetry_beacon'):
                    return self._m_telemetry_beacon

                self._m_telemetry_beacon = self.telemetry + self.counter_ascii + u"," + self.analog1_ascii + u"," + self.analog2_ascii + u"," + self.analog3_ascii + u"," + self.analog4_ascii + u"," + self.analog5_ascii + u"," + self.digital1_ascii + self.digital1_ascii + self.digital3_ascii + self.digital4_ascii + self.digital5_ascii + self.digital6_ascii + self.digital7_ascii + self.digital8_ascii
                return getattr(self, '_m_telemetry_beacon', None)

            @property
            def digital2(self):
                if hasattr(self, '_m_digital2'):
                    return self._m_digital2

                self._m_digital2 = int(self.digital2_ascii)
                return getattr(self, '_m_digital2', None)

            @property
            def digital1(self):
                if hasattr(self, '_m_digital1'):
                    return self._m_digital1

                self._m_digital1 = int(self.digital1_ascii)
                return getattr(self, '_m_digital1', None)

            @property
            def analog3(self):
                if hasattr(self, '_m_analog3'):
                    return self._m_analog3

                self._m_analog3 = int(self.analog3_ascii)
                return getattr(self, '_m_analog3', None)

            @property
            def digital8(self):
                if hasattr(self, '_m_digital8'):
                    return self._m_digital8

                self._m_digital8 = int(self.digital8_ascii)
                return getattr(self, '_m_digital8', None)

            @property
            def analog5(self):
                if hasattr(self, '_m_analog5'):
                    return self._m_analog5

                self._m_analog5 = int(self.analog5_ascii)
                return getattr(self, '_m_analog5', None)

            @property
            def digital7(self):
                if hasattr(self, '_m_digital7'):
                    return self._m_digital7

                self._m_digital7 = int(self.digital7_ascii)
                return getattr(self, '_m_digital7', None)

            @property
            def analog2(self):
                if hasattr(self, '_m_analog2'):
                    return self._m_analog2

                self._m_analog2 = int(self.analog2_ascii)
                return getattr(self, '_m_analog2', None)



    class Digi(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_frame = Io86.Digi.Ax25Frame(self._io, self, self._root)

        class Ax25Frame(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.ax25_header = Io86.Digi.Ax25Header(self._io, self, self._root)
                _on = (self.ax25_header.ctl & 19)
                if _on == 0:
                    self.payload = Io86.Digi.IFrame(self._io, self, self._root)
                elif _on == 3:
                    self.payload = Io86.Digi.UiFrame(self._io, self, self._root)
                elif _on == 19:
                    self.payload = Io86.Digi.UiFrame(self._io, self, self._root)
                elif _on == 16:
                    self.payload = Io86.Digi.IFrame(self._io, self, self._root)
                elif _on == 18:
                    self.payload = Io86.Digi.IFrame(self._io, self, self._root)
                elif _on == 2:
                    self.payload = Io86.Digi.IFrame(self._io, self, self._root)


        class Ax25Header(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.dest_callsign_raw = Io86.Digi.CallsignRaw(self._io, self, self._root)
                self.dest_ssid_raw = Io86.Digi.SsidMask(self._io, self, self._root)
                self.src_callsign_raw = Io86.Digi.CallsignRaw(self._io, self, self._root)
                self.src_ssid_raw = Io86.Digi.SsidMask(self._io, self, self._root)
                if (self.src_ssid_raw.ssid_mask & 1) == 0:
                    self.repeater = Io86.Digi.Repeater(self._io, self, self._root)

                self.ctl = self._io.read_u1()


        class UiFrame(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.pid = self._io.read_u1()
                self._raw_ax25_info = self._io.read_bytes_full()
                _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
                self.ax25_info = Io86.Digi.Ax25InfoData(_io__raw_ax25_info, self, self._root)


        class Callsign(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


        class IFrame(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.pid = self._io.read_u1()
                self._raw_ax25_info = self._io.read_bytes_full()
                _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
                self.ax25_info = Io86.Digi.Ax25InfoData(_io__raw_ax25_info, self, self._root)


        class SsidMask(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.ssid_mask = self._io.read_u1()

            @property
            def ssid(self):
                if hasattr(self, '_m_ssid'):
                    return self._m_ssid

                self._m_ssid = ((self.ssid_mask & 15) >> 1)
                return getattr(self, '_m_ssid', None)

            @property
            def hbit(self):
                if hasattr(self, '_m_hbit'):
                    return self._m_hbit

                self._m_hbit = ((self.ssid_mask & 128) >> 7)
                return getattr(self, '_m_hbit', None)


        class Repeaters(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.rpt_callsign_raw = Io86.Digi.CallsignRaw(self._io, self, self._root)
                self.rpt_ssid_raw = Io86.Digi.SsidMask(self._io, self, self._root)


        class Repeater(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.rpt_instance = []
                i = 0
                while True:
                    _ = Io86.Digi.Repeaters(self._io, self, self._root)
                    self.rpt_instance.append(_)
                    if (_.rpt_ssid_raw.ssid_mask & 1) == 1:
                        break
                    i += 1


        class CallsignRaw(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self._raw__raw_callsign_ror = self._io.read_bytes(6)
                self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
                _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
                self.callsign_ror = Io86.Digi.Callsign(_io__raw_callsign_ror, self, self._root)


        class Ax25InfoData(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.data_monitor = (self._io.read_bytes_full()).decode(u"utf-8")







# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO
from enum import Enum


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Suchai2(KaitaiStruct):
    """:field prio: sch.csp_h.prio
    :field source: sch.csp_h.source
    :field dest: sch.csp_h.dest
    :field dest_port: sch.csp_h.dest_port
    :field source_port: sch.csp_h.source_port
    :field flags: sch.csp_h.flags
    :field nframe: sch.nframe
    :field type: sch.type
    :field node: sch.node
    :field ndata: sch.ndata
    :field index: sch.status.index
    :field timestamp: sch.status.timestamp
    :field dat_obc_opmode: sch.status.dat_obc_opmode
    :field dat_rtc_date_time: sch.status.dat_rtc_date_time
    :field dat_obc_last_reset: sch.status.dat_obc_last_reset
    :field dat_obc_hrs_alive: sch.status.dat_obc_hrs_alive
    :field dat_obc_hrs_wo_reset: sch.status.dat_obc_hrs_wo_reset
    :field dat_obc_reset_counter: sch.status.dat_obc_reset_counter
    :field dat_obc_executed_cmds: sch.status.dat_obc_executed_cmds
    :field dat_obc_failed_cmds: sch.status.dat_obc_failed_cmds
    :field dat_com_count_tm: sch.status.dat_com_count_tm
    :field dat_com_count_tc: sch.status.dat_com_count_tc
    :field dat_com_last_tc: sch.status.dat_com_last_tc
    :field dat_fpl_last: sch.status.dat_fpl_last
    :field dat_fpl_queue: sch.status.dat_fpl_queue
    :field dat_ads_tle_epoch: sch.status.dat_ads_tle_epoch
    :field dat_eps_vbatt: sch.status.dat_eps_vbatt
    :field dat_eps_cur_sun: sch.status.dat_eps_cur_sun
    :field dat_eps_cur_sys: sch.status.dat_eps_cur_sys
    :field dat_obc_temp_1: sch.status.dat_obc_temp_1
    :field dat_eps_temp_bat0: sch.status.dat_eps_temp_bat0
    :field dat_drp_mach_action: sch.status.dat_drp_mach_action
    :field dat_drp_mach_state: sch.status.dat_drp_mach_state
    :field dat_drp_mach_payloads: sch.status.dat_drp_mach_payloads
    :field dat_drp_mach_step: sch.status.dat_drp_mach_step
    :field sat_id: sch.sat_id
    """

    class SatName(Enum):
        suchai2 = 16
        suchai3 = 17
        plantsat = 18
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.sch = Suchai2.SuchaiFrame(self._io, self, self._root)

    class SuchaiFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.csp_h = Suchai2.CspHeader(self._io, self, self._root)
            self.nframe = self._io.read_u2be()
            self.type = self._io.read_u1()
            self.node = self._io.read_u1()
            self.ndata = self._io.read_u4be()
            self.status = Suchai2.Status(self._io, self, self._root)

        @property
        def sat_id(self):
            if hasattr(self, '_m_sat_id'):
                return self._m_sat_id

            self._m_sat_id = KaitaiStream.resolve_enum(Suchai2.SatName, self.csp_h.dest_port)
            return getattr(self, '_m_sat_id', None)


    class CspHeader(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.prio = self._io.read_bits_int_be(2)
            self.source = self._io.read_bits_int_be(5)
            self.dest = self._io.read_bits_int_be(5)
            self.dest_port = self._io.read_bits_int_be(6)
            self.source_port = self._io.read_bits_int_be(6)
            self._io.align_to_byte()
            self.flags = self._io.read_u1()


    class Status(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.index = self._io.read_u4be()
            self.timestamp = self._io.read_u4be()
            self.dat_obc_opmode = self._io.read_u4be()
            self.dat_rtc_date_time = self._io.read_u4be()
            self.dat_obc_last_reset = self._io.read_u4be()
            self.dat_obc_hrs_alive = self._io.read_u4be()
            self.dat_obc_hrs_wo_reset = self._io.read_u4be()
            self.dat_obc_reset_counter = self._io.read_u4be()
            self.dat_obc_executed_cmds = self._io.read_u4be()
            self.dat_obc_failed_cmds = self._io.read_u4be()
            self.dat_com_count_tm = self._io.read_u4be()
            self.dat_com_count_tc = self._io.read_u4be()
            self.dat_com_last_tc = self._io.read_u4be()
            self.dat_fpl_last = self._io.read_u4be()
            self.dat_fpl_queue = self._io.read_u4be()
            self.dat_ads_tle_epoch = self._io.read_u4be()
            self.dat_eps_vbatt = self._io.read_u4be()
            self.dat_eps_cur_sun = self._io.read_u4be()
            self.dat_eps_cur_sys = self._io.read_u4be()
            self.dat_obc_temp_1 = self._io.read_u4be()
            self.dat_eps_temp_bat0 = self._io.read_u4be()
            self.dat_drp_mach_action = self._io.read_u4be()
            self.dat_drp_mach_state = self._io.read_u4be()
            self.dat_drp_mach_payloads = self._io.read_u4be()
            self.dat_drp_mach_step = self._io.read_u4be()






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Cevrosat(KaitaiStruct):
    """:field uptime_total: id_one.type_check.uptime_total
    :field uptime_since_last: id_one.type_check.uptime_since_last
    :field reset_count: id_one.type_check.reset_count
    :field mcu_10mv: id_one.type_check.mcu_10mv
    :field batt: id_one.type_check.batt
    :field temp_cpu: id_one.type_check.temp_cpu
    :field temp_pa_ntc: id_one.type_check.temp_pa_ntc
    :field sig_rx_immediate: id_one.type_check.sig_rx_immediate
    :field sig_rx_avg: id_one.type_check.sig_rx_avg
    :field sig_rx_max: id_one.type_check.sig_rx_max
    :field sig_background_avg: id_one.type_check.sig_background_avg
    :field sig_background_immediate: id_one.type_check.sig_background_immediate
    :field sig_background_max: id_one.type_check.sig_background_max
    :field rf_packets_received: id_one.type_check.rf_packets_received
    :field rf_packets_transmitted: id_one.type_check.rf_packets_transmitted
    :field ax25_packets_received: id_one.type_check.ax25_packets_received
    :field ax25_packets_transmitted: id_one.type_check.ax25_packets_transmitted
    :field digipeater_rx_count: id_one.type_check.digipeater_rx_count
    :field digipeater_tx_count: id_one.type_check.digipeater_tx_count
    :field csp_received: id_one.type_check.csp_received
    :field csp_transmitted: id_one.type_check.csp_transmitted
    :field i2c1_received: id_one.type_check.i2c1_received
    :field i2c1_transmitted: id_one.type_check.i2c1_transmitted
    :field i2c2_received: id_one.type_check.i2c2_received
    :field i2c2_transmitted: id_one.type_check.i2c2_transmitted
    :field rs485_received: id_one.type_check.rs485_received
    :field rs485_transmitted: id_one.type_check.rs485_transmitted
    :field csp_mcu_received: id_one.type_check.csp_mcu_received
    :field csp_mcu_transmitted: id_one.type_check.csp_mcu_transmitted
    :field a: id_one.type_check.a
    :field tx1_telemetry: id_one.type_check.tx1_telemetry
    :field src_callsign: id_one.type_check.ax25_frame.ax25_header.dnxd_src_callsign_raw.callsign_ror.callsign
    :field src_ssid: id_one.type_check.ax25_frame.ax25_header.dnxd_src_ssid_raw.ssid
    :field dest_callsign: id_one.type_check.ax25_frame.ax25_header.dnxd_dest_callsign_raw.callsign_ror.callsign
    :field dest_ssid: id_one.type_check.ax25_frame.ax25_header.dnxd_dest_ssid_raw.ssid
    :field dnxd_message: id_one.type_check.ax25_frame.dnxd_message
    :field src_callsign: id_one.type_check.digi_ax25_frame.digi_ax25_header.digi_src_callsign_raw.callsign_ror.callsign
    :field src_ssid: id_one.type_check.digi_ax25_frame.digi_ax25_header.digi_src_ssid_raw.ssid
    :field dest_callsign: id_one.type_check.digi_ax25_frame.digi_ax25_header.digi_dest_callsign_raw.callsign_ror.callsign
    :field dest_ssid: id_one.type_check.digi_ax25_frame.digi_ax25_header.digi_dest_ssid_raw.ssid
    :field rpt_instance_callsign: id_one.type_check.digi_ax25_frame.digi_ax25_header.repeater.rpt_instance.rpt_callsign_raw.callsign_ror.callsign
    :field rpt_instance_ssid: id_one.type_check.digi_ax25_frame.digi_ax25_header.repeater.rpt_instance.rpt_ssid_raw.ssid
    :field digi_message: id_one.type_check.digi_ax25_frame.digi_message
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.id_one = Cevrosat.IdOneType(self._io, self, self._root)

    class IdOneType(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            _on = self.check
            if _on == 2259461217:
                self.type_check = Cevrosat.Tx1(self._io, self, self._root)
            elif _on == 2259461233:
                self.type_check = Cevrosat.Dnxd(self._io, self, self._root)
            else:
                self.type_check = Cevrosat.Digi(self._io, self, self._root)

        @property
        def check(self):
            if hasattr(self, '_m_check'):
                return self._m_check

            _pos = self._io.pos()
            self._io.seek(10)
            self._m_check = self._io.read_u4be()
            self._io.seek(_pos)
            return getattr(self, '_m_check', None)


    class Tx1(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.skip_ax25_header_1 = self._io.read_u8be()
            if not self.skip_ax25_header_1 == 9701387192009547934:
                raise kaitaistruct.ValidationNotEqualError(9701387192009547934, self.skip_ax25_header_1, self._io, u"/types/tx_1/seq/0")
            self.skip_ax25_header_2 = self._io.read_u8be()
            if not self.skip_ax25_header_2 == 10835808779503731696:
                raise kaitaistruct.ValidationNotEqualError(10835808779503731696, self.skip_ax25_header_2, self._io, u"/types/tx_1/seq/1")
            self.first_comma = self._io.read_u1()
            if not self.first_comma == 44:
                raise kaitaistruct.ValidationNotEqualError(44, self.first_comma, self._io, u"/types/tx_1/seq/2")
            self.tx_1 = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            if not self.tx_1 == u"TX-1":
                raise kaitaistruct.ValidationNotEqualError(u"TX-1", self.tx_1, self._io, u"/types/tx_1/seq/3")
            self.pass_uptime = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            if not self.pass_uptime == u"U":
                raise kaitaistruct.ValidationNotEqualError(u"U", self.pass_uptime, self._io, u"/types/tx_1/seq/4")
            self.uptime_total_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.uptime_since_last_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.pass_resets = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            if not self.pass_resets == u"R":
                raise kaitaistruct.ValidationNotEqualError(u"R", self.pass_resets, self._io, u"/types/tx_1/seq/7")
            self.reset_count_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.pass_mcuv = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            if not self.pass_mcuv == u"V":
                raise kaitaistruct.ValidationNotEqualError(u"V", self.pass_mcuv, self._io, u"/types/tx_1/seq/9")
            self.mcu_10mv_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.pass_battv = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            if not self.pass_battv == u"Ve":
                raise kaitaistruct.ValidationNotEqualError(u"Ve", self.pass_battv, self._io, u"/types/tx_1/seq/11")
            self.batt_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.pass_temp = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            if not self.pass_temp == u"T":
                raise kaitaistruct.ValidationNotEqualError(u"T", self.pass_temp, self._io, u"/types/tx_1/seq/13")
            self.temp_cpu_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.temp_pa_ntc_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.pass_sig = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            if not self.pass_sig == u"Sig":
                raise kaitaistruct.ValidationNotEqualError(u"Sig", self.pass_sig, self._io, u"/types/tx_1/seq/16")
            self.sig_rx_immediate_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.sig_rx_avg_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.sig_rx_max_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.sig_background_immediate_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.sig_background_avg_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.sig_background_max_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.pass_rf = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            if not self.pass_rf == u"RX":
                raise kaitaistruct.ValidationNotEqualError(u"RX", self.pass_rf, self._io, u"/types/tx_1/seq/23")
            self.rf_packets_received_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.rf_packets_transmitted_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.pass_ax25 = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            if not self.pass_ax25 == u"Ax":
                raise kaitaistruct.ValidationNotEqualError(u"Ax", self.pass_ax25, self._io, u"/types/tx_1/seq/26")
            self.ax25_packets_received_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.ax25_packets_transmitted_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.pass_digi = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            if not self.pass_digi == u"Digi":
                raise kaitaistruct.ValidationNotEqualError(u"Digi", self.pass_digi, self._io, u"/types/tx_1/seq/29")
            self.digipeater_rx_count_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.digipeater_tx_count_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.pass_csp = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            if not self.pass_csp == u"CSP":
                raise kaitaistruct.ValidationNotEqualError(u"CSP", self.pass_csp, self._io, u"/types/tx_1/seq/32")
            self.csp_received_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.csp_transmitted_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.pass_i2c1 = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            if not self.pass_i2c1 == u"I2C1":
                raise kaitaistruct.ValidationNotEqualError(u"I2C1", self.pass_i2c1, self._io, u"/types/tx_1/seq/35")
            self.i2c1_received_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.i2c1_transmitted_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.pass_i2c2 = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            if not self.pass_i2c2 == u"I2C2":
                raise kaitaistruct.ValidationNotEqualError(u"I2C2", self.pass_i2c2, self._io, u"/types/tx_1/seq/38")
            self.i2c2_received_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.i2c2_transmitted_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.pass_rs485 = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            if not self.pass_rs485 == u"RS485":
                raise kaitaistruct.ValidationNotEqualError(u"RS485", self.pass_rs485, self._io, u"/types/tx_1/seq/41")
            self.rs485_received_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.rs485_transmitted_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.pass_csp_mcu = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            if not self.pass_csp_mcu == u"MCU":
                raise kaitaistruct.ValidationNotEqualError(u"MCU", self.pass_csp_mcu, self._io, u"/types/tx_1/seq/44")
            self.csp_mcu_received_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.csp_mcu_transmitted_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.pass_a = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            if not self.pass_a == u"A":
                raise kaitaistruct.ValidationNotEqualError(u"A", self.pass_a, self._io, u"/types/tx_1/seq/47")
            self.a_raw = (self._io.read_bytes_full()).decode(u"utf8")

        @property
        def sig_rx_max(self):
            if hasattr(self, '_m_sig_rx_max'):
                return self._m_sig_rx_max

            self._m_sig_rx_max = int(self.sig_rx_max_raw)
            return getattr(self, '_m_sig_rx_max', None)

        @property
        def temp_pa_ntc(self):
            if hasattr(self, '_m_temp_pa_ntc'):
                return self._m_temp_pa_ntc

            self._m_temp_pa_ntc = int(self.temp_pa_ntc_raw)
            return getattr(self, '_m_temp_pa_ntc', None)

        @property
        def csp_transmitted(self):
            if hasattr(self, '_m_csp_transmitted'):
                return self._m_csp_transmitted

            self._m_csp_transmitted = int(self.csp_transmitted_raw)
            return getattr(self, '_m_csp_transmitted', None)

        @property
        def batt(self):
            if hasattr(self, '_m_batt'):
                return self._m_batt

            self._m_batt = int(self.batt_raw)
            return getattr(self, '_m_batt', None)

        @property
        def sig_rx_avg(self):
            if hasattr(self, '_m_sig_rx_avg'):
                return self._m_sig_rx_avg

            self._m_sig_rx_avg = int(self.sig_rx_avg_raw)
            return getattr(self, '_m_sig_rx_avg', None)

        @property
        def sig_background_immediate(self):
            if hasattr(self, '_m_sig_background_immediate'):
                return self._m_sig_background_immediate

            self._m_sig_background_immediate = int(self.sig_background_immediate_raw)
            return getattr(self, '_m_sig_background_immediate', None)

        @property
        def uptime_total(self):
            if hasattr(self, '_m_uptime_total'):
                return self._m_uptime_total

            self._m_uptime_total = int(self.uptime_total_raw)
            return getattr(self, '_m_uptime_total', None)

        @property
        def rs485_received(self):
            if hasattr(self, '_m_rs485_received'):
                return self._m_rs485_received

            self._m_rs485_received = int(self.rs485_received_raw)
            return getattr(self, '_m_rs485_received', None)

        @property
        def i2c1_received(self):
            if hasattr(self, '_m_i2c1_received'):
                return self._m_i2c1_received

            self._m_i2c1_received = int(self.i2c1_received_raw)
            return getattr(self, '_m_i2c1_received', None)

        @property
        def a(self):
            if hasattr(self, '_m_a'):
                return self._m_a

            self._m_a = int(self.a_raw)
            return getattr(self, '_m_a', None)

        @property
        def temp_cpu(self):
            if hasattr(self, '_m_temp_cpu'):
                return self._m_temp_cpu

            self._m_temp_cpu = int(self.temp_cpu_raw)
            return getattr(self, '_m_temp_cpu', None)

        @property
        def ax25_packets_transmitted(self):
            if hasattr(self, '_m_ax25_packets_transmitted'):
                return self._m_ax25_packets_transmitted

            self._m_ax25_packets_transmitted = int(self.ax25_packets_transmitted_raw)
            return getattr(self, '_m_ax25_packets_transmitted', None)

        @property
        def ax25_packets_received(self):
            if hasattr(self, '_m_ax25_packets_received'):
                return self._m_ax25_packets_received

            self._m_ax25_packets_received = int(self.ax25_packets_received_raw)
            return getattr(self, '_m_ax25_packets_received', None)

        @property
        def digipeater_tx_count(self):
            if hasattr(self, '_m_digipeater_tx_count'):
                return self._m_digipeater_tx_count

            self._m_digipeater_tx_count = int(self.digipeater_tx_count_raw)
            return getattr(self, '_m_digipeater_tx_count', None)

        @property
        def csp_mcu_transmitted(self):
            if hasattr(self, '_m_csp_mcu_transmitted'):
                return self._m_csp_mcu_transmitted

            self._m_csp_mcu_transmitted = int(self.csp_mcu_transmitted_raw)
            return getattr(self, '_m_csp_mcu_transmitted', None)

        @property
        def tx1_telemetry(self):
            if hasattr(self, '_m_tx1_telemetry'):
                return self._m_tx1_telemetry

            self._m_tx1_telemetry = u",TX-1,U," + self.uptime_total_raw + u"," + self.uptime_since_last_raw + u",R," + self.reset_count_raw + u",V," + self.mcu_10mv_raw + u",Ve," + self.batt_raw + u",T," + self.temp_cpu_raw + u"," + self.temp_pa_ntc_raw + u",Sig," + self.sig_rx_immediate_raw + u"," + self.sig_rx_avg_raw + u"," + self.sig_rx_max_raw + u"," + self.sig_background_immediate_raw + u"," + self.sig_background_avg_raw + u"," + self.sig_background_max_raw + u",RX," + self.rf_packets_received_raw + u"," + self.rf_packets_transmitted_raw + u",Ax," + self.ax25_packets_received_raw + u"," + self.ax25_packets_transmitted_raw + u",Digi," + self.digipeater_rx_count_raw + u"," + self.digipeater_tx_count_raw + u",CSP," + self.csp_received_raw + u"," + self.csp_transmitted_raw + u",I2C1," + self.i2c1_received_raw + u"," + self.i2c1_transmitted_raw + u",I2C2," + self.i2c2_received_raw + u"," + self.i2c2_received_raw + u",RS485," + self.rs485_received_raw + u"," + self.rs485_transmitted_raw + u",MCU," + self.csp_mcu_received_raw + u"," + self.csp_mcu_transmitted_raw + u",A," + self.a_raw
            return getattr(self, '_m_tx1_telemetry', None)

        @property
        def csp_mcu_received(self):
            if hasattr(self, '_m_csp_mcu_received'):
                return self._m_csp_mcu_received

            self._m_csp_mcu_received = int(self.csp_mcu_received_raw)
            return getattr(self, '_m_csp_mcu_received', None)

        @property
        def i2c1_transmitted(self):
            if hasattr(self, '_m_i2c1_transmitted'):
                return self._m_i2c1_transmitted

            self._m_i2c1_transmitted = int(self.i2c1_transmitted_raw)
            return getattr(self, '_m_i2c1_transmitted', None)

        @property
        def mcu_10mv(self):
            if hasattr(self, '_m_mcu_10mv'):
                return self._m_mcu_10mv

            self._m_mcu_10mv = int(self.mcu_10mv_raw)
            return getattr(self, '_m_mcu_10mv', None)

        @property
        def uptime_since_last(self):
            if hasattr(self, '_m_uptime_since_last'):
                return self._m_uptime_since_last

            self._m_uptime_since_last = int(self.uptime_since_last_raw)
            return getattr(self, '_m_uptime_since_last', None)

        @property
        def sig_background_max(self):
            if hasattr(self, '_m_sig_background_max'):
                return self._m_sig_background_max

            self._m_sig_background_max = int(self.sig_background_max_raw)
            return getattr(self, '_m_sig_background_max', None)

        @property
        def sig_rx_immediate(self):
            if hasattr(self, '_m_sig_rx_immediate'):
                return self._m_sig_rx_immediate

            self._m_sig_rx_immediate = int(self.sig_rx_immediate_raw)
            return getattr(self, '_m_sig_rx_immediate', None)

        @property
        def reset_count(self):
            if hasattr(self, '_m_reset_count'):
                return self._m_reset_count

            self._m_reset_count = int(self.reset_count_raw)
            return getattr(self, '_m_reset_count', None)

        @property
        def rs485_transmitted(self):
            if hasattr(self, '_m_rs485_transmitted'):
                return self._m_rs485_transmitted

            self._m_rs485_transmitted = int(self.rs485_transmitted_raw)
            return getattr(self, '_m_rs485_transmitted', None)

        @property
        def rf_packets_received(self):
            if hasattr(self, '_m_rf_packets_received'):
                return self._m_rf_packets_received

            self._m_rf_packets_received = int(self.rf_packets_received_raw)
            return getattr(self, '_m_rf_packets_received', None)

        @property
        def rf_packets_transmitted(self):
            if hasattr(self, '_m_rf_packets_transmitted'):
                return self._m_rf_packets_transmitted

            self._m_rf_packets_transmitted = int(self.rf_packets_transmitted_raw)
            return getattr(self, '_m_rf_packets_transmitted', None)

        @property
        def digipeater_rx_count(self):
            if hasattr(self, '_m_digipeater_rx_count'):
                return self._m_digipeater_rx_count

            self._m_digipeater_rx_count = int(self.digipeater_rx_count_raw)
            return getattr(self, '_m_digipeater_rx_count', None)

        @property
        def sig_background_avg(self):
            if hasattr(self, '_m_sig_background_avg'):
                return self._m_sig_background_avg

            self._m_sig_background_avg = int(self.sig_background_avg_raw)
            return getattr(self, '_m_sig_background_avg', None)

        @property
        def i2c2_received(self):
            if hasattr(self, '_m_i2c2_received'):
                return self._m_i2c2_received

            self._m_i2c2_received = int(self.i2c2_received_raw)
            return getattr(self, '_m_i2c2_received', None)

        @property
        def i2c2_transmitted(self):
            if hasattr(self, '_m_i2c2_transmitted'):
                return self._m_i2c2_transmitted

            self._m_i2c2_transmitted = int(self.i2c2_transmitted_raw)
            return getattr(self, '_m_i2c2_transmitted', None)

        @property
        def csp_received(self):
            if hasattr(self, '_m_csp_received'):
                return self._m_csp_received

            self._m_csp_received = int(self.csp_received_raw)
            return getattr(self, '_m_csp_received', None)


    class Dnxd(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_frame = Cevrosat.Dnxd.Ax25Frame(self._io, self, self._root)

        class Ax25Frame(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.ax25_header = Cevrosat.Dnxd.Ax25Header(self._io, self, self._root)
                self.dnxd_message = (self._io.read_bytes_full()).decode(u"utf-8")


        class Ax25Header(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.dnxd_dest_callsign_raw = Cevrosat.Dnxd.CallsignRaw(self._io, self, self._root)
                self.dnxd_dest_ssid_raw = Cevrosat.Dnxd.SsidMask(self._io, self, self._root)
                self.dnxd_src_callsign_raw = Cevrosat.Dnxd.CallsignRaw(self._io, self, self._root)
                if self.dnxd_src_callsign_raw.callsign_ror.callsign == u"OK0CVR":
                    self.dnxd_src_ssid_raw = Cevrosat.Dnxd.SsidMask(self._io, self, self._root)

                if self.dnxd_src_ssid_raw.ssid == 8:
                    self.ctl_pid = self._io.read_u2be()



        class Callsign(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


        class SsidMask(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.ssid_mask = self._io.read_u1()

            @property
            def ssid(self):
                if hasattr(self, '_m_ssid'):
                    return self._m_ssid

                self._m_ssid = ((self.ssid_mask & 31) >> 1)
                return getattr(self, '_m_ssid', None)


        class CallsignRaw(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self._raw__raw_callsign_ror = self._io.read_bytes(6)
                self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
                _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
                self.callsign_ror = Cevrosat.Dnxd.Callsign(_io__raw_callsign_ror, self, self._root)



    class Digi(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.digi_ax25_frame = Cevrosat.Digi.Ax25Frame(self._io, self, self._root)

        class Ax25Frame(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.digi_ax25_header = Cevrosat.Digi.Ax25Header(self._io, self, self._root)
                self.digi_message = (self._io.read_bytes_full()).decode(u"utf-8")


        class Ax25Header(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.digi_dest_callsign_raw = Cevrosat.Digi.CallsignRaw(self._io, self, self._root)
                self.digi_dest_ssid_raw = Cevrosat.Digi.SsidMask(self._io, self, self._root)
                self.digi_src_callsign_raw = Cevrosat.Digi.CallsignRaw(self._io, self, self._root)
                self.digi_src_ssid_raw = Cevrosat.Digi.SsidMask(self._io, self, self._root)
                if (self.digi_src_ssid_raw.ssid_mask & 1) == 0:
                    self.repeater = Cevrosat.Digi.Repeater(self._io, self, self._root)

                if self.repeater.rpt_instance.rpt_callsign_raw.callsign_ror.callsign == u"OK0CVR":
                    self.ctl = self._io.read_u1()

                if  ((self.repeater.rpt_instance.rpt_ssid_raw.ssid == 7) or (self.repeater.rpt_instance.rpt_ssid_raw.ssid == 8)) :
                    self.pid = self._io.read_u1()



        class Callsign(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


        class SsidMask(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.ssid_mask = self._io.read_u1()

            @property
            def ssid(self):
                if hasattr(self, '_m_ssid'):
                    return self._m_ssid

                self._m_ssid = ((self.ssid_mask & 31) >> 1)
                return getattr(self, '_m_ssid', None)


        class Repeaters(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.rpt_callsign_raw = Cevrosat.Digi.CallsignRaw(self._io, self, self._root)
                self.rpt_ssid_raw = Cevrosat.Digi.SsidMask(self._io, self, self._root)


        class Repeater(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.rpt_instance = Cevrosat.Digi.Repeaters(self._io, self, self._root)


        class CallsignRaw(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self._raw__raw_callsign_ror = self._io.read_bytes(6)
                self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
                _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
                self.callsign_ror = Cevrosat.Digi.Callsign(_io__raw_callsign_ror, self, self._root)




